-- Pulls back the Addon-Local Variables and store them locally.
local addonName, addonTable = ...;
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

local debugprofilestop, format, print, tinsert = debugprofilestop, format, print, tinsert;

local profileLog = {};
local enableProfiling = false;

-- This is stored on the addon so its publically available to all other addons.
addon.Core.Profiling = {
	Log = function()
		if enableProfiling then
			addonTable.accountSettings.Profiling = addon.Core.Tables.Duplicate(profileLog);

			print("Profiling log now saved, please reload UI to finish writing to saved variables!");
		end
	end,

	New = function(functionName, ID)
		local self = {};
		local endTime, startTime;

		self.Start = function()
			if enableProfiling and functionName then
				startTime = debugprofilestop();
			end
		end

		self.Stop = function()
			if enableProfiling and startTime and functionName and ID then
				local action = addon.Icons.AOE.Actions()[1];

				endTime = debugprofilestop();

--				tinsert(profileLog, format("[%s] - Function: %s, Time Elapsed: %s ms", ID, functionName, endTime - startTime));
				if action then
					tinsert(profileLog, format("[%s], Index: %s, Spell: %s, %s", ID, action.RotationIndex, action.Name, endTime - startTime));
				end
			end
		end

		return self;
	end,

	Toggle = function()
		enableProfiling = not enableProfiling;

		print(format("Profiling is now %s!", enableProfiling and "enabled" or "disabled"));
	end
};