-- Pulls back the Addon-Local Variables and store them locally.
local addonName, addonTable = ...;
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

local Blizzard = addon.API.Blizzard;
local format, gsub, ipairs, pairs, select, sort, strlen, strmatch, strupper, tinsert, tonumber, tremove, wipe = format, gsub, ipairs, pairs, select, sort, strlen, strmatch, strupper, tinsert, addon.Core.General.ToNumber, tremove, addon.Core.Tables.Wipe;

local buttonMapping, keybinds = {}, {};
local actionBar = {
	Item = {},
	Spell = {},
};
addon.Core.Tables.CaseInsensitive(actionBar);

-- Addon Inspecific Wrapper Functions
local Actions, Buttons, Keybinds, Process, Scan = {}, {}, {}, {}, {};
-- Addon Specific Functions
local Bartender4, Default, Dominos, ElvUI = {}, {}, {}, {};

--------------------- Addon Specific Functions ---------------------
Bartender4 = {
	Active = function(buttonID)
		return buttonID >= 1 and buttonID <= 120;
	end,

	Button = function(buttonID)
		local buttonName = Bartender4.ButtonName(buttonID);

		return buttonName ~= nil and _G[buttonName];
	end,

	ButtonName = function(buttonID)
		buttonID = tonumber(buttonID);

		return format("BT4Button%i", buttonID);
	end,

	Keybind = function(buttonID)
		local defaultName = Default.KeybindName(buttonID);
		local keybindName = Bartender4.KeybindName(buttonID);

		if keybindName then
			if buttonID <= 12 and defaultName then
				return Blizzard.GetBindingKey(keybindName) or Blizzard.GetBindingKey(defaultName);
			end

			return Blizzard.GetBindingKey(keybindName);
		end
	end,

	KeybindName = function(buttonID)
		buttonID = tonumber(buttonID);

		return format("CLICK %s:LeftButton", Bartender4.ButtonName(buttonID));
	end,
};

Default = {
	Active = function(buttonID)
		if buttonID <= 24 or buttonID >= 73 then
			local currentPage = Blizzard.GetActionBarPage();
			local bonusBarIndex = Blizzard.GetBonusBarIndex();
			local newID = addon.Core.Maths.mod(buttonID - 1, 12) + 1;

			if Blizzard.HasBonusActionBar() and bonusBarIndex ~= 0 then
				currentPage = bonusBarIndex;
			end

			return buttonID == ((currentPage - 1) * 12) + newID;
		else
			return true;
		end
	end,

	Button = function(buttonID)
		local buttonName = Default.ButtonName(buttonID);

		return buttonName ~= nil and _G[buttonName];
	end,

	ButtonName = function(buttonID)
		if buttonID <= 24 then
			return format("ActionButton%i", addon.Core.Maths.mod(buttonID - 1, 12) + 1);
		elseif buttonID <= 36 then
			return format("MultiBarRightButton%i", buttonID - 24);
		elseif buttonID <= 48 then
			return format("MultiBarLeftButton%i", buttonID - 36);
		elseif buttonID <= 60 then
			return format("MultiBarBottomRightButton%i", buttonID - 48);
		elseif buttonID <= 72 then
			return format("MultiBarBottomLeftButton%i", buttonID - 60);
		else
			return format("ActionButton%i", addon.Core.Maths.mod(buttonID - 1, 12) + 1);
		end
	end,

	Keybind = function(buttonID)
		local keybindName = Default.KeybindName(buttonID);

		return keybindName ~= nil and Blizzard.GetBindingKey(keybindName);
	end,

	KeybindName = function(buttonID)
		if buttonID <= 24 then
			return format("ACTIONBUTTON%i", addon.Core.Maths.mod(buttonID - 1, 12) + 1);
		elseif buttonID <= 36 then
			return format("MULTIACTIONBAR3BUTTON%i", buttonID - 24);
		elseif buttonID <= 48 then
			return format("MULTIACTIONBAR4BUTTON%i", buttonID - 36);
		elseif buttonID <= 60 then
			return format("MULTIACTIONBAR2BUTTON%i", buttonID - 48);
		elseif buttonID <= 72 then
			return format("MULTIACTIONBAR1BUTTON%i", buttonID - 60);
		else
			return format("ACTIONBUTTON%i", addon.Core.Maths.mod(buttonID - 1, 12) + 1);
		end
	end,
};

Dominos = {
	Active = function(buttonID)
		return buttonID >= 1 and buttonID <= 120;
	end,

	Button = function(buttonID)
		local buttonName = Dominos.ButtonName(buttonID);

		return buttonName ~= nil and _G[buttonName];
	end,

	ButtonName = function(buttonID)
		if buttonID >= 13 and buttonID <= 24 then
			return format("DominosActionButton%i", buttonID - 12);
		elseif buttonID >= 73 and buttonID <= 120 then
			return format("DominosActionButton%i", buttonID - 60);
		else
			return Default.ButtonName(buttonID);
		end
	end,

	Keybind = function(buttonID)
		local keybindName = Dominos.KeybindName(buttonID);

		return keybindName ~= nil and Blizzard.GetBindingKey(keybindName);
	end,

	KeybindName = function(buttonID)
		if (buttonID >= 13 and buttonID <= 24) or (buttonID >= 73 and buttonID <= 120) then
			return format("CLICK %s:LeftButton", Dominos.ButtonName(buttonID));
		else
			return Default.KeybindName(buttonID);
		end
	end,
};

ElvUI = {
	Active = function(buttonID)
		if buttonID <= 12 or buttonID >= 73 then
			local currentPage = Blizzard.GetActionBarPage();
			local bonusBarIndex = Blizzard.GetBonusBarIndex();
			local newID = addon.Core.Maths.mod(buttonID - 1, 12) + 1;

			if Blizzard.HasBonusActionBar() and bonusBarIndex ~= 0 then
				currentPage = bonusBarIndex;
			end

			return buttonID == ((currentPage - 1) * 12) + newID;
		else
			return true;
		end
	end,

	Button = function(buttonID)
		local buttonName = ElvUI.ButtonName(buttonID);

		return buttonName ~= nil and _G[buttonName];
	end,

	ButtonName = function(buttonID)
		if buttonID <= 12 then
			return format("ElvUI_Bar1Button%i", buttonID);
		elseif buttonID <= 24 then
			return format("ElvUI_Bar6Button%i", buttonID - 12);
		elseif buttonID <= 36 then
			return format("ElvUI_Bar5Button%i", buttonID - 24);
		elseif buttonID <= 48 then
			return format("ElvUI_Bar4Button%i", buttonID - 36);
		elseif buttonID <= 60 then
			return format("ElvUI_Bar2Button%i", buttonID - 48);
		elseif buttonID <= 72 then
			return format("ElvUI_Bar3Button%i", buttonID - 60);
		else
			return format("ElvUI_Bar1Button%i", addon.Core.Maths.mod(buttonID - 1, 12) + 1);
		end
	end,

	Keybind = function(buttonID)
		local keybindName = ElvUI.KeybindName(buttonID);

		return keybindName ~= nil and Blizzard.GetBindingKey(keybindName);
	end,

	KeybindName = function(buttonID)
		if buttonID >= 13 and buttonID <= 24 then
			return format("ELVUIBAR6BUTTON%i", buttonID - 12);
		else
			return Default.KeybindName(buttonID);
		end
	end,
};

------------------------ Wrapper Functions  ------------------------
Actions = {
	Add = function(buttonID, actionType, actionID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and actionID and not Actions.Exists(buttonID, actionType, actionID) then
			local Tables = addon.Core.Tables;
			local actionTable = Tables.GetValue(actionBar[actionType], actionID, {});

			tinsert(actionTable, buttonID);
			sort(actionTable);

			actionBar[actionType][actionID] = actionTable;
			buttonMapping[buttonID] = {
				actionType = actionType,
				actionID = actionID,
			};

			return true;
		end

		return false;
	end,

	Changed = function(buttonID, actionType, actionID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and actionID then
			local Tables = addon.Core.Tables;

			-- Get the previously stored information in that slotID.
			local previousType = Tables.GetValue(buttonMapping[buttonID], "actionType", "");
			local previousID = Tables.GetValue(buttonMapping[buttonID], "actionID", 0);

			return actionType ~= previousType or actionID ~= previousID, previousType, previousID;
		end

		return false;
	end,

	Exists = function(buttonID, actionType, actionID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and actionID then
			local Tables = addon.Core.Tables;
			local actionTable = Tables.GetValue(actionBar[actionType], actionID, {});

			if not Tables.IsEmpty(actionTable) then
				for index, data in ipairs(actionTable) do
					if data == buttonID then
						return true, index;
					end
				end
			end
		end

		return false;
	end,

	Remove = function(buttonID, actionType, actionID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and actionID then
			local buttonFound, buttonIndex = Actions.Exists(buttonID, actionType, actionID);

			if buttonFound then
				local Tables = addon.Core.Tables;
				local actionTable = Tables.GetValue(actionBar[actionType], actionID, {});

				tremove(actionTable, buttonIndex);
				sort(actionTable);

				actionBar[actionType][actionID] = actionTable;
				buttonMapping[buttonID] = nil;

				return true;
			end
		end

		return false;
	end,
};

Buttons = {
	Active = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID >= 1 and buttonID <= 120 then
			if Blizzard.IsAddOnLoaded("ElvUI") then
				return ElvUI.Active(buttonID);
			elseif Blizzard.IsAddOnLoaded("Bartender4") then
				return Bartender4.Active(buttonID);
			elseif Blizzard.IsAddOnLoaded("Dominos") then
				return Dominos.Active(buttonID);
			else
				return Default.Active(buttonID);
			end
		end

		return false;
	end,

	Get = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID >= 1 and buttonID <= 120 then
			if Blizzard.IsAddOnLoaded("ElvUI") then
				return ElvUI.Button(buttonID);
			elseif Blizzard.IsAddOnLoaded("Bartender4") then
				return Bartender4.Button(buttonID);
			elseif Blizzard.IsAddOnLoaded("Dominos") then
				return Dominos.Button(buttonID);
			else
				return Default.Button(buttonID);
			end
		end
	end,

	Name = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID >= 1 and buttonID <= 120 then
			if Blizzard.IsAddOnLoaded("ElvUI") then
				return ElvUI.ButtonName(buttonID);
			elseif Blizzard.IsAddOnLoaded("Bartender4") then
				return Bartender4.ButtonName(buttonID);
			elseif Blizzard.IsAddOnLoaded("Dominos") then
				return Dominos.ButtonName(buttonID);
			else
				return Default.ButtonName(buttonID);
			end
		end
	end,
};

Keybinds = {
	Get = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID >= 1 and buttonID <= 120 then
			local returnValue;

			if Blizzard.IsAddOnLoaded("ElvUI") then
				returnValue = ElvUI.Keybind(buttonID);
			elseif Blizzard.IsAddOnLoaded("Bartender4") then
				returnValue = Bartender4.Keybind(buttonID);
			elseif Blizzard.IsAddOnLoaded("Dominos") then
				returnValue = Dominos.Keybind(buttonID);
			else
				returnValue = Default.Keybind(buttonID);
			end

			return Process.Keybind(returnValue) or "";
		end

		return "";
	end,

	Name = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID >= 1 and buttonID <= 120 then
			if Blizzard.IsAddOnLoaded("ElvUI") then
				return ElvUI.KeybindName(buttonID);
			elseif Blizzard.IsAddOnLoaded("Bartender4") then
				return Bartender4.KeybindName(buttonID);
			elseif Blizzard.IsAddOnLoaded("Dominos") then
				return Dominos.KeybindName(buttonID);
			else
				return Default.KeybindName(buttonID);
			end
		end
	end,
};

Process = {
	Action = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID then
			-- Map the current action in the slotID.
			local actionType, actionID = Blizzard.GetActionInfo(buttonID);

			if actionType == "spell" then
				Process.Spell(buttonID, actionType, actionID);
			elseif actionType == "item" then
				Process.Item(buttonID, actionType, actionID);
			elseif actionType == "macro" then
				Process.Macro(buttonID, actionType, actionID);
			else
				local actionChanged, previousType, previousID = Actions.Changed(buttonID, actionType, actionID);

				-- We have no action information so we are only removing the old existing data if its changed.
				if actionChanged then
					Actions.Remove(buttonID, previousType, previousID);
				end
			end
		end
	end,

	Item = function(buttonID, actionType, actionID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and actionID then
			local actionChanged, previousType, previousID = Actions.Changed(buttonID, actionType, actionID);

			if actionChanged then
				-- Clear previous bindings for this slotID
				Actions.Remove(buttonID, previousType, previousID);
				Actions.Add(buttonID, actionType, actionID);
			end
		end
	end,

	Keybind = function(keybind)
		if keybind and keybind ~= "" then
			-- Shorten the keybinding names.
			if keybind and strlen(keybind) > 4 then
				-- Convert to Uppercase
				keybind = strupper(keybind);
				-- Strip whitespace.
				keybind = gsub(keybind, "%s+", "");
				-- Convert modifiers to a single character.
				keybind = gsub(keybind, "ALT%-", "A-");
				keybind = gsub(keybind, "CTRL%-", "C-");
				keybind = gsub(keybind, "SHIFT%-", "S-");
				-- Shorten number pad keybinding names.
				keybind = gsub(keybind, "NUMPAD", "N");
				keybind = gsub(keybind, "PLUS", "+");
				keybind = gsub(keybind, "MINUS", "-");
				keybind = gsub(keybind, "MULTIPLY", "*");
				keybind = gsub(keybind, "DIVIDE", "/");
				keybind = gsub(keybind, "SPACE", "SP");
				-- Shorten mouse keybinding names.
				keybind = gsub(keybind, "BUTTON", "MB");
			end

			return keybind or "";
		end

		return "";
	end,

	Macro = function(buttonID, actionType, macroID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and macroID and Blizzard.GetActionText(buttonID) ~= nil then
			local actionID = Blizzard.GetMacroSpell(macroID);

			if actionID then
				Process.Spell(buttonID, "spell", actionID);
			else
				local hyperlink = select(2, Blizzard.GetMacroItem(macroID));

				if hyperlink then
					local linkData = select(3, strmatch(hyperlink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+)|?h?%[?([^%[%] ]*)%]?|?h?|?r?"));
					local replacedString = gsub(linkData, ":.*", "");

					actionID = tonumber(replacedString);

					if actionID then
						Process.Item(buttonID, "item", actionID);
					end
				end
			end
		end
	end,

	Spell = function(buttonID, actionType, actionID)
		buttonID = tonumber(buttonID);

		if buttonID and actionType and actionID then
			local actionChanged, previousType, previousID = Actions.Changed(buttonID, actionType, actionID);

			if actionChanged then
				-- Clear previous bindings for this slotID
				Actions.Remove(buttonID, previousType, previousID);
				Actions.Add(buttonID, actionType, actionID);
			end
		end
	end,
};

Scan = {
	ActionBar = function()
		wipe(buttonMapping);
		wipe(actionBar.item);
		wipe(actionBar.spell);

		for buttonID = 1, 120 do
			Process.Action(buttonID);
		end
	end,

	ActionButton = function(...)
		local _, buttonID = ...;

		if buttonID == 0 then
			Scan.ActionBar();
		else
			Process.Action(buttonID);
		end
	end,

	Bindings = function()
		wipe(keybinds);

		for buttonID = 1, 120 do
			keybinds[buttonID] = Keybinds.Get(buttonID);
		end
	end
};

-- This is stored on the addon so its publically available to all other addons.
addon.Modules.ActionBar = {
	Active = function(buttonID)
		return Buttons.Active(buttonID);
	end,

	Button = function(buttonID)
		buttonID = tonumber(buttonID);

		if buttonID then
			return Buttons.Get(buttonID);
		end

		return nil;
	end,

	ButtonID = function(actionType, actionID)
		if actionType and actionID then
			local Tables = addon.Core.Tables;
			local actionTable = Tables.GetValue(actionBar[actionType], actionID, {});

			if not Tables.IsEmpty(actionTable) then
				for _, data in ipairs(actionTable) do
					if Buttons.Active(data) then
						return data;
					end
				end
			end
		end

		return nil;
	end,

	Keybind = function(actionType, actionID)
		if actionType and actionID then
			local coreObjects = addon.Core.Objects;
			local Tables = addon.Core.Tables;

			local buttonID = addon.Modules.ActionBar.ButtonID(actionType, actionID);
			local keybind;

			if actionType == "Spell" then
				local currentSpell = addonTable.getSpell(actionID);

				if coreObjects.IsSpell(currentSpell) then
					keybind = addonTable.Prediction.SpellKeybind(currentSpell.ClassID());
				end
			end

			return keybind or Tables.GetValue(keybinds, buttonID, "");
		end

		return "";
	end,
};

local moduleEvents = {
	playerLogin = function()
		Scan.ActionBar();
		Scan.Bindings();
	end,

	spellsChanged = function()
		Scan.ActionBar();
	end,

	updateMacros = function()
		Scan.ActionBar();
	end,
};

local tableEvents = {
	ACTIONBAR_SLOT_CHANGED = Scan.ActionBar,
	UPDATE_BINDINGS = Scan.Bindings,
	UPDATE_MACROS = Scan.ActionBar,
};

addon.System.Callbacks.RegisterAddonCallbacks(moduleEvents);
addon.System.Events.RegisterEvents(tableEvents, true);