-- Pulls back the Addon-Local Variables and store them locally.
local addonName, addonTable = ...;
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

-- Store local copy of Blizzard API wrapper and other addon global functions and variables
local Callbacks = addon.System.Callbacks;
local huge, loadstring, max, pairs, tinsert, tremove, type, unpack = math.huge, loadstring, max, pairs, tinsert, tremove, type, unpack;

local eliteMobs = {
	elite = true,
	rareelite = true,
};

local normalMobs = {
	minus = true,
	normal = true,
	trivial = true,
};

local stealthSpells = {
	[1784] = true, -- Stealth
	[5215] = true, -- Prowl
	[1856] = true, -- Vanish
	[58984] = true, -- Shadowmeld
};

local subrotationList = {};
local subrotationProcessing = {};
local rotationIndex = 0;

-- Icons
addon.Icons = {
	AOE = addonTable.iconRotation("MultiTarget"),
	CD = addonTable.iconRotation("Cooldown"),
	OFFGCD = addonTable.iconRotation("OffGCD"),
	ST = addonTable.iconRotation("SingleTarget"),
	DEFENSIVE = addonTable.iconDefensive,
	INTERRUPT = {},
};

local variablesRotation, indexTable = {}, {};

local function evalutateCheckBox(checkbox)
	local Settings = addon.Core.Settings;

	return (checkbox == nil and 2)
	    or (type(checkbox) == "boolean" and checkbox == true and 2)
	    or (type(checkbox) == "boolean" and checkbox == false and 1)
	    or Settings.GetCharacterValue("ScriptOptions", checkbox)
	    or Settings.GetRotationValue(checkbox);
end

local function setEnemies(numEnemies)
	local Enemies = addon.Modules.Enemies;
	local multiTarget = addon.Icons.AOE;

	if multiTarget.Paused() or multiTarget.Stopped() then
		return 1;
	end

	return Enemies.GetEnemies(numEnemies);
end

local function actionCooldown(Object, useObject, numEnemies)
	local Objects = addon.Core.Objects;

	local currentRotation = subrotationList[#subrotationList] or addon.currentRotation;
	local CD = addon.Icons.CD;

	local useSingleTarget, useAOE = unpack(subrotationProcessing[#subrotationProcessing]);
	local enemyCount = setEnemies(numEnemies);

	return evalutateCheckBox(Object.CheckBox()) == 1
	   and not CD.Paused()
	   and not CD.Stopped()
	   and Objects.IsSpell(Object)
	   and not Object.IsOffGCD()
	   and ((enemyCount == 1 and useSingleTarget) or useAOE)
	   and currentRotation.ProcessCondition(useObject, setEnemies(numEnemies));
end

local function actionMultiTarget(Object, useObject, numEnemies)
	local Enemies = addon.Modules.Enemies;

	local currentRotation = subrotationList[#subrotationList] or addon.currentRotation;
	local multiTarget = addon.Icons.AOE;
	local _, useAOE = unpack(subrotationProcessing[#subrotationProcessing]);

	return evalutateCheckBox(Object.CheckBox()) == 2
	   and not multiTarget.Paused()
	   and not multiTarget.Stopped()
	   and useAOE
	   and currentRotation.ProcessCondition(useObject, Enemies.GetEnemies(numEnemies));
end

local function actionOffGCD(Object, useObject, numEnemies)
	local Objects = addon.Core.Objects;

	local currentRotation = subrotationList[#subrotationList] or addon.currentRotation;
	local offGCD = addon.Icons.OFFGCD;

	local useSingleTarget, useAOE = unpack(subrotationProcessing[#subrotationProcessing]);
	local enemyCount = setEnemies(numEnemies);

	return evalutateCheckBox(Object.CheckBox()) == 1
	   and not offGCD.Paused()
	   and not offGCD.Stopped()
	   and (Objects.IsItem(Object) or (Objects.IsSpell(Object) and Object.IsOffGCD()))
	   and ((enemyCount == 1 and useSingleTarget) or useAOE)
	   and currentRotation.ProcessCondition(useObject, setEnemies(numEnemies));
end

local function actionSingleTarget(Object, useObject)
	local currentRotation = subrotationList[#subrotationList] or addon.currentRotation;
	local singleTarget = addon.Icons.ST;
	local useSingleTarget = unpack(subrotationProcessing[#subrotationProcessing]);

	return evalutateCheckBox(Object.CheckBox()) == 2
	   and not singleTarget.Paused()
	   and not singleTarget.Stopped()
	   and useSingleTarget
	   and currentRotation.ProcessCondition(useObject, 1);
end

local function stopProcessing()
	local singleTarget = addon.Icons.ST;
	local multiTarget =	addon.Icons.AOE;

	return (singleTarget.Paused() or singleTarget.Stopped())
	   and (multiTarget.Paused() or multiTarget.Stopped());
end

local function multiTargetRotation(Object, callFunction, numEnemies)
	local multiTarget =	addon.Icons.AOE;
	local _, prevUse = unpack(subrotationProcessing[#subrotationProcessing]);

	return not multiTarget.Paused()
	   and not multiTarget.Stopped()
	   and prevUse
	   and Object.ProcessRotation(callFunction, addon.Modules.Enemies.GetEnemies(numEnemies));
end

local function singleTargetRotation(Object, callFunction)
	local singleTarget = addon.Icons.ST;
	local prevUse = unpack(subrotationProcessing[#subrotationProcessing]);

	return not singleTarget.Paused()
	   and not singleTarget.Stopped()
	   and prevUse
	   and Object.ProcessRotation(callFunction, 1);
end

local function useAlways(evaluateAction)
	local useSolo = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useSolo") == 1;
	local baseCooldown = addon.Core.Tables.GetValue(evaluateAction, "BaseCooldown", 0);

	return baseCooldown <= 15 or addon.Core.General.IsRaidBossFight() or (useSolo and addon.Units.Player.IsSolo());
end

local function useDungeonBoss(evaluateAction)
	local useDungeon = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useDungeonBoss") == 1;

	local baseCooldown = addon.Core.Tables.GetValue(evaluateAction, "BaseCooldown", 0);
	local maxCooldown = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useDungeonBossMax");

	return addon.Core.General.IsDungeonBossFight() and (not useDungeon or baseCooldown <= maxCooldown);
end

local function useEliteMob(Target, evaluateAction)
	local eliteMob = eliteMobs[Target.Classification()] or false;
	local useElite = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useElite") == 1;

	local baseCooldown = addon.Core.Tables.GetValue(evaluateAction, "BaseCooldown", 0);
	local maxCooldown = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useEliteMax");

	return eliteMob and (not useElite or baseCooldown <= maxCooldown);
end

local function useNormalMob(Target, evaluateAction)
	local normalMob = normalMobs[Target.Classification()] or false;
	local useNormal = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useNormal") == 1;

	local baseCooldown = addon.Core.Tables.GetValue(evaluateAction, "BaseCooldown", 0);
	local maxCooldown = addon.Core.Settings.GetCharacterValue("CooldownOptions", "useNormalMax");

	return normalMob and (not useNormal or baseCooldown <= maxCooldown);
end

local function usePVP(Target, evaluateAction)
	local PVP = addon.Core.Settings.GetCharacterValue("CooldownOptions", "usePVP") == 1;

	local baseCooldown = addon.Core.Tables.GetValue(evaluateAction, "BaseCooldown", 0);
	local maxCooldown = addon.Core.Settings.GetCharacterValue("CooldownOptions", "usePVPMax");

	return Target.IsPlayer() and (not PVP or baseCooldown <= maxCooldown);
end

local function useStealth(Object)
	return not addon.API.Unit.InCombat()
	    or addon.Core.Settings.GetCharacterValue("ScriptOptions", "OPT_STEALTH_SOLO") == 1
	    or addon.Units.Player.InGroup()
	    or not addon.Core.Objects.IsSpell(Object)
	    or not stealthSpells[Object.ClassID()];
end

local function validWaitTime(evaluateAction)
	local Settings = addon.Core.Settings;
	local Tables = addon.Core.Tables;

	local currentTime = addon.API.Blizzard.GetTime();
	local useMaxWait = Settings.GetAccountValue("AddonSettings", "useMaxWait") == 1;
	local maxWaitTime = Settings.GetAccountValue("AddonSettings", "maxWaitTime", 10);

	return not useMaxWait or Tables.GetValue(evaluateAction, "WaitEnd", huge) - currentTime <= maxWaitTime;
end

local function useCooldown(evaluateAction, Target)
	local currentTarget = Target or addon.Units.Target;

	return (evaluateAction ~= nil and evaluateAction ~= false)
	   and validWaitTime(evaluateAction)
	   and (useAlways(evaluateAction) or useDungeonBoss(evaluateAction) or useEliteMob(currentTarget, evaluateAction) or useNormalMob(currentTarget, evaluateAction) or usePVP(currentTarget, evaluateAction));
end

local function validObjects(Object, useObject)
	local Objects = addon.Core.Objects;

	return ((Objects.IsSpell(Object) and Object.IsKnown()) or (Objects.IsItem(Object) and Object.Equipped()))
	   and (useObject == true or type(useObject) == "function");
end

local function evaluateAction(Object, useObject, numEnemies, iconText, Target, interruptCheck)
	rotationIndex = rotationIndex + 1;

	if not stopProcessing() and validObjects(Object, useObject) then
		-- Todo: Add code to handle simcraft interrupt channel condition correctly.
		local evaluate = Object.Evaluate();

		if useCooldown(evaluate, Target) and useStealth(Object) then
			if actionSingleTarget(Object, useObject) then
				addon.Icons.ST.EvaluateAction(rotationIndex, Object, iconText, evaluate);
			end

			if actionMultiTarget(Object, useObject, numEnemies) then
				addon.Icons.AOE.EvaluateAction(rotationIndex, Object, iconText, evaluate);
			end

			if actionOffGCD(Object, useObject, numEnemies) then
				addon.Icons.OFFGCD.EvaluateAction(rotationIndex, Object, iconText, evaluate);
			end

			if actionCooldown(Object, useObject, numEnemies) then
				addon.Icons.CD.EvaluateAction(rotationIndex, Object, iconText, evaluate);
			end
		end
	end
end

addonTable.RotationProcessing = {
	CallActionList = function(Object, callFunction, numEnemies)
		if not stopProcessing() then
			local useSingleTarget = singleTargetRotation(Object, callFunction);
			local useMultiTarget = multiTargetRotation(Object, callFunction, numEnemies);

			tinsert(subrotationList, Object);
			tinsert(subrotationProcessing, { useSingleTarget, useMultiTarget });

			if useSingleTarget or useMultiTarget then
				Object.Rotation(addonTable.RotationProcessing);
			end

			subrotationList[#subrotationList] = nil;
			subrotationProcessing[#subrotationProcessing] = nil;
		end
	end,

	EvaluateAction = function(Object, useObject, numEnemies, iconText)
		evaluateAction(Object, useObject, numEnemies, iconText);
	end,

	EvaluateCycleAction = function(Object, useObject, maxTargets, numEnemies, iconText)
		evaluateAction(Object, useObject, numEnemies, iconText);

		-- Todo: Add multi-dot part of this action.
		-- Only multi-dot if this action if still processing AOE actions, or there is multiple enemies to dot.
		if numEnemies or addon.Enemies > 1 then
		end
	end,

	EvaluateInterruptCondition = function(Object, useObject, interruptCheck, numEnemies, iconText)
		evaluateAction(Object, useObject, numEnemies, iconText, nil, interruptCheck);
	end,

	EvaluatePoolAction = function(Object, useObject, numEnemies, iconText)
		-- Todo: Add code to handle simcraft pool resources condition correctly.
		evaluateAction(Object, useObject, numEnemies, iconText);
	end,

	EvaluateWaitCondition = function(Object, waitCheck, numEnemies)
		rotationIndex = rotationIndex + 1;

		-- Todo: Add code to handle simcraft wait condition correctly.
		if not stopProcessing() and validObjects(Object, waitCheck) then
			if actionSingleTarget(Object, waitCheck) then
				addon.Icons.ST.EvaluateWaitCondition(rotationIndex, Object);
			end

			if actionMultiTarget(Object, waitCheck, numEnemies) then
				addon.Icons.AOE.EvaluateWaitCondition(rotationIndex, Object);
			end
		end
	end,

	GetValue = function(name, default)
		if addon.currentRotation and name then
			local coreTables = addon.Core.Tables;

			local rotationName = addon.currentRotation.Name();
			local rotationTable = coreTables.GetValue(variablesRotation, rotationName, {});

			return coreTables.GetValue(rotationTable, name, default);
		end

		return default;
	end,
	
	RunActionList = function(Object, callFunction, numEnemies)
		if not stopProcessing() and Object then
			local useSingleTarget = singleTargetRotation(Object, callFunction);
			local useMultiTarget = multiTargetRotation(Object, callFunction, numEnemies);

			tinsert(subrotationList, Object);
			tinsert(subrotationProcessing, { useSingleTarget, useMultiTarget });

			if useSingleTarget or useMultiTarget then
				Object.Rotation(addonTable.RotationProcessing);

				if useSingleTarget then
					addon.Icons.ST.Stopped(useSingleTarget);
				end

				if useMultiTarget then
					addon.Icons.AOE.Stopped(useMultiTarget);
				end
			end

			subrotationList[#subrotationList] = nil;
			subrotationProcessing[#subrotationProcessing] = nil;
		end
	end,

	SetValue = function(name, value)
		if addon.currentRotation and name and type(value) == "function" then
			local rotationName = addon.currentRotation.Name();

			variablesRotation[rotationName] = variablesRotation[rotationName] or {};
			variablesRotation[rotationName][name] = value();

			return true;
		end

		return false;
	end,
};

addon.Rotation = {
	GetActiveRotation = function()
		local General = addon.API.General;
		local Rotation = addon.Rotation;
		local Tables = addon.Core.Tables;

		local profile = Tables.GetValue(addonTable.characterSettings, "ActiveRotations", {});
		local specInfo = General.GetSpecializationInfo();
		local returnValue;

		if not Tables.IsEmpty(specInfo) then
			returnValue = Tables.GetValue(profile, specInfo.Name);

			if not returnValue then
				local numRotations, rotationList = Rotation.GetRegisteredRotations(specInfo.ID);

				if not Tables.IsEmpty(rotationList) then
					local index = 0;
					local lowestIndex = huge;

					for key in pairs(rotationList) do
						index = Tables.GetKey(indexTable, key, huge);
						lowestIndex = index < lowestIndex and index or lowestIndex;
					end

					returnValue = indexTable[lowestIndex];

					profile[specInfo.Name] = returnValue;
				else
					returnValue = "__none";
				end

				-- Put the settings into the database
			 	addonTable.characterSettings.ActiveRotations = profile;
			end
		end

		return returnValue;
	end,

	GetRegisteredRotations = function(classSpec)
		if classSpec then
			local Tables = addon.Core.Tables;
			local specRotations = addonTable.RegisteredRotations[classSpec];

			if specRotations and Tables.IsTable(specRotations) then
				return Tables.Count(specRotations), specRotations;
			end
		end

		return 0, nil;
	end,

	GetRotation = function(classSpec, rotationName)
		if classSpec and rotationName ~= "__none" then
			local Rotation = addon.Rotation;
			local Tables = addon.Core.Tables;

			local numRotations, rotationList = Rotation.GetRegisteredRotations(classSpec);

			rotationName = rotationName or Rotation.GetActiveRotation();

			if not Tables.IsEmpty(rotationList) and rotationName ~= nil then
				return Tables.GetValue(rotationList[rotationName], "callback");
			end
		end

		return nil;
	end,

	RegisterRotation = function(Rotation)
		local Tables = addon.Core.Tables;

		if Rotation then
			local classSpec = Tables.GetValue(Rotation, "SpecID", 0);
			local rotationName = Tables.GetValue(Rotation, "Name", "");
			local rotationDescription = Tables.GetValue(Rotation, "Description", "");

			local specRotations = addonTable.RegisteredRotations[classSpec] or {};

			specRotations[rotationName] = {
				callback = Rotation,
				description = rotationDescription,
			};

			addonTable.RegisteredRotations[classSpec] = specRotations;
			tinsert(indexTable, rotationName);
		end
	end,

	SetActiveRotation = function(rotationName)
		if rotationName then
			local General = addon.API.General;
			local Tables = addon.Core.Tables;

			local specInfo = General.GetSpecializationInfo();

			if not Tables.IsEmpty(specInfo) then
				local profile = Tables.GetValue(addonTable.characterSettings, "ActiveRotations", {});

				profile[specInfo.Name] = rotationName;

				-- Put the settings into the database
				addonTable.characterSettings.ActiveRotations = profile;

				-- Send message so active rotation is changed to the selected one.
				Callbacks.Callback("rotationChanged");
			end
		end
	end,

	StartRotation = function()
		rotationIndex = 0;
		tinsert(subrotationList, addon.currentRotation);
		tinsert(subrotationProcessing, { true, true });
	end,

	UnregisterRotation = function(Rotation)
		local Tables = addon.Core.Tables;

		if Rotation then
			local classSpec = Tables.GetValue(Rotation, "SpecID", 0);
			local rotationName = Tables.GetValue(Rotation, "Name", "");
			local specRotations = addonTable.RegisteredRotations[classSpec];
			local index = Tables.GetKey(specRotations, rotationName);

			addonTable.RegisteredRotations[classSpec][rotationName] = nil;
			tremove(indexTable, index);
		end
	end,
};