local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

local min = min;

--- Localize Vars
local coreSettings = addon.Core.Settings;
local Enemies = addon.Modules.Enemies;
local Objects = addon.Core.Objects;

local getVar = addonTable.RotationProcessing.GetValue;

-- Objects
local Pet = addon.Units.Pet;
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "lunaeclipse_hunter_survival";

-- Cleave Rotation
local function Cleave(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.cleave=variable,name=carve_cdr,op=setif,value=active_enemies,value_else=5,condition=active_enemies<5
		action.SetValue("carve_cdr", function() return min(Enemies.GetEnemies(), 5); end);
		-- actions.cleave+=/a_murder_of_crows
		action.EvaluateAction(Talent.AMurderOfCrows, true);
		-- actions.cleave+=/coordinated_assault
		action.EvaluateAction(Spell.CoordinatedAssault, true);
		-- actions.cleave+=/carve,if=dot.shrapnel_bomb.ticking
		action.EvaluateAction(Spell.Carve, function()
			return Target.Debuff(Aura.ShrapnelBomb).Ticking();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions so we need three extra copies of this line.
		-- actions.cleave+=/wildfire_bomb,if=!talent.guerrilla_tactics.enabled|full_recharge_time<gcd
		action.EvaluateAction(Spell.WildfireBomb, function()
			return not Talent.GuerrillaTactics.Enabled()
			    or Spell.WildfireBomb.Charges.FullRechargeTime() < Player.GCD();
		end);
		action.EvaluateAction(Spell.PheromoneBomb, function()
			return not Talent.GuerrillaTactics.Enabled()
			    or Spell.PheromoneBomb.Charges.FullRechargeTime() < Player.GCD();
		end);
		action.EvaluateAction(Spell.ShrapnelBomb, function()
			return not Talent.GuerrillaTactics.Enabled()
			    or Spell.ShrapnelBomb.Charges.FullRechargeTime() < Player.GCD();
		end);
		action.EvaluateAction(Spell.VolatileBomb, function()
			return not Talent.GuerrillaTactics.Enabled()
			    or Spell.VolatileBomb.Charges.FullRechargeTime() < Player.GCD();
		end);
		-- actions.cleave+=/mongoose_bite,target_if=max:debuff.latent_poison.stack,if=debuff.latent_poison.stack=10
		action.EvaluateCycleAction(Talent.MongooseBite, function(_, Target)
			return Target.Debuff(Aura.LatentPoison).Stack() == 10;
		end);
		-- actions.cleave+=/chakrams
		action.EvaluateAction(Talent.Chakrams, true);
		-- actions.cleave+=/kill_command,target_if=min:bloodseeker.remains,if=focus+cast_regen<focus.max
		action.EvaluateAction(Spell.KillCommand, function()
			return Player.Focus() + Player.Focus.CastRegen(Spell.KillCommand) < Player.Focus.Max();
		end);
		-- actions.cleave+=/butchery,if=full_recharge_time<gcd|!talent.wildfire_infusion.enabled|dot.shrapnel_bomb.ticking&dot.internal_bleeding.stack<3
		action.EvaluateAction(Talent.Butchery, function(_, Target)
			return Talent.Butchery.Charges.FullRechargeTime() < Player.GCD()
			    or not Talent.WildfireInfusion.Enabled()
			    or Target.Debuff(Aura.ShrapnelBomb).Ticking()
			   and Target.Debuff(Aura.InternalBleeding).Stack() < 3;
		end);
		-- actions.cleave+=/carve,if=talent.guerrilla_tactics.enabled
		action.EvaluateAction(Spell.Carve, function()
			return Talent.GuerrillaTactics.Enabled();
		end);
		-- actions.cleave+=/flanking_strike,if=focus+cast_regen<focus.max
		action.EvaluateAction(Talent.FlankingStrike, function()
			return Player.Focus() + Player.Focus.CastRegen(Talent.FlankingStrike) < Player.Focus.Max();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions so we need three extra copies of this line.
		-- actions.cleave+=/wildfire_bomb,if=dot.wildfire_bomb.refreshable|talent.wildfire_infusion.enabled
		action.EvaluateAction(Spell.WildfireBomb, function()
			return Target.Debuff(Aura.WildfireBomb).Refreshable()
			    or Talent.WildfireInfusion.Enabled();
		end);
		action.EvaluateAction(Spell.PheromoneBomb, function()
			return Target.Debuff(Aura.WildfireBomb).Refreshable()
			    or Talent.WildfireInfusion.Enabled();
		end);
		action.EvaluateAction(Spell.ShrapnelBomb, function()
			return Target.Debuff(Aura.WildfireBomb).Refreshable()
			    or Talent.WildfireInfusion.Enabled();
		end);
		action.EvaluateAction(Spell.VolatileBomb, function()
			return Target.Debuff(Aura.WildfireBomb).Refreshable()
			    or Talent.WildfireInfusion.Enabled();
		end);
		-- actions.cleave+=/serpent_sting,target_if=min:remains,if=buff.vipers_venom.up
		action.EvaluateCycleAction(Spell.SerpentSting, function()
			return Player.Buff(Aura.VipersVenom).Up();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions so we need to check these cooldowns as well.
		-- actions.cleave+=/carve,if=cooldown.wildfire_bomb.remains>variable.carve_cdr%2
		action.EvaluateAction(Spell.Carve, function()
			return coreSettings.GetRotationValue(Spell.WildfireBomb.CheckBox(), 2) ~= 2
			    or Spell.WildfireBomb.Cooldown.Remains() > getVar("carve_cdr", 1) / 2
			    or Spell.PheromoneBomb.Cooldown.Remains() > getVar("carve_cdr", 1) / 2
			    or Spell.ShrapnelBomb.Cooldown.Remains() > getVar("carve_cdr", 1) / 2
			    or Spell.VolatileBomb.Cooldown.Remains() > getVar("carve_cdr", 1) / 2;
		end);
		-- actions.cleave+=/steel_trap
		action.EvaluateAction(Talent.SteelTrap, true);
		-- actions.cleave+=/harpoon,if=talent.terms_of_engagement.enabled
		action.EvaluateAction(Spell.Harpoon, function()
			return Talent.TermsOfEngagement.Enabled();
		end);
		-- actions.cleave+=/serpent_sting,target_if=min:remains,if=refreshable&buff.tip_of_the_spear.stack<3
		action.EvaluateCycleAction(Spell.SerpentSting, function(_, Target)
			return Target.Debuff(Aura.SerpentSting).Refreshable()
			   and Player.Buff(Aura.TipOfTheSpear).Stack() < 3;
		end);
		-- # To simulate usage for Mongoose Bite or Raptor Strike during Aspect of the Eagle, copy each occurrence of the action and append _eagle to the action name.
		-- actions.cleave+=/mongoose_bite,target_if=max:debuff.latent_poison.stack
		action.EvaluateCycleAction(Talent.MongooseBite, function(_, Target)
			return Target.Debuff(Aura.LatentPoison).Up();
		end);
		-- actions.cleave+=/raptor_strike,target_if=max:debuff.latent_poison.stack
		action.EvaluateCycleAction(Spell.RaptorStrike, function(_, Target)
			return Target.Debuff(Aura.LatentPoison).Up();
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationCleave = Cleave("Cleave");

-- Cooldowns Rotation
local function Cooldowns(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.cds=blood_fury,if=cooldown.coordinated_assault.remains>30
		action.EvaluateAction(Racial.BloodFury, function()
			return Spell.CoordinatedAssault.Cooldown.Remains() > 30
			    or coreSettings.GetRotationValue(Spell.CoordinatedAssault.CheckBox(), 2) ~= 2;
		end);
		-- actions.cds+=/ancestral_call,if=cooldown.coordinated_assault.remains>30
		action.EvaluateAction(Racial.AncestralCall, function()
			return Spell.CoordinatedAssault.Cooldown.Remains() > 30
			    or coreSettings.GetRotationValue(Spell.CoordinatedAssault.CheckBox(), 2) ~= 2;
		end);
		-- actions.cds+=/fireblood,if=cooldown.coordinated_assault.remains>30
		action.EvaluateAction(Racial.Fireblood, function()
			return Spell.CoordinatedAssault.Cooldown.Remains() > 30
			    or coreSettings.GetRotationValue(Spell.CoordinatedAssault.CheckBox(), 2) ~= 2;
		end);
		-- actions.cds+=/lights_judgment
		action.EvaluateAction(Racial.LightsJudgment, true);
		-- actions.cds+=/berserking,if=cooldown.coordinated_assault.remains>60|time_to_die<11
		action.EvaluateAction(Racial.Berserking, function()
			return (Spell.CoordinatedAssault.Cooldown.Remains() > 60 or coreSettings.GetRotationValue(Spell.CoordinatedAssault.CheckBox(), 2) ~= 2)
			    or Target.TimeToDie() < 11;
		end);

		-- actions.cds+=/potion,if=buff.coordinated_assault.up&(buff.berserking.up|buff.blood_fury.up|!race.troll&!race.orc)|time_to_die<26
		action.EvaluateAction(Consumable.BattlePotionOfAgility, function()
			return Player.Buff(Aura.CoordinatedAssault).Up()
			   and (Player.Buff(Aura.Berserking).Up() or Player.Buff(Aura.BloodFury).Up() or (not Player.Race("Troll") and not Player.Race("Orc")))
			    or Target.TimeToDie() < 26;
		end);
		-- actions.cds+=/aspect_of_the_eagle,if=target.distance>=6
		action.EvaluateAction(Spell.AspectOfTheEagle, function()
			return not Target.InRange(6);
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationCooldowns = Cooldowns("Cooldowns");

-- MongooseBite Rotation
local function MongooseBite(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.mb_ap_wfi_st=serpent_sting,if=!dot.serpent_sting.ticking
		action.EvaluateAction(Spell.SerpentSting, function()
			return not Target.Debuff(Aura.SerpentSting).Ticking();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions, this is the pheromone and volatile bomb parts.
		-- We don't need to do the next_wi_bomb part, as the addon does this automatically as it won't suggest the spell if its not in the spellbook.
		-- actions.mb_ap_wfi_st+=/wildfire_bomb,if=full_recharge_time<gcd|(focus+cast_regen<focus.max)&(next_wi_bomb.volatile&dot.serpent_sting.ticking&dot.serpent_sting.refreshable|next_wi_bomb.pheromone&!buff.mongoose_fury.up&focus+cast_regen<focus.max-action.kill_command.cast_regen*3)
		action.EvaluateAction(Spell.PheromoneBomb, function()
			return Spell.PheromoneBomb.Charges.FullRechargeTime() < Player.GCD()
			    or (Player.Focus() + Player.Focus.CastRegen(Spell.PheromoneBomb) < Player.Focus.Max())
			   and (not Player.Buff(Aura.MongooseFury).Up() and Player.Focus() + Player.Focus.CastRegen(Spell.PheromoneBomb) < Player.Focus.Max() - Player.Focus.CastRegen(Spell.KillCommand) * 3);
		end);
		action.EvaluateAction(Spell.VolatileBomb, function()
			return Spell.VolatileBomb.Charges.FullRechargeTime() < Player.GCD()
			    or (Player.Focus() + Player.Focus.CastRegen(Spell.VolatileBomb) < Player.Focus.Max())
			   and (Target.Debuff(Aura.SerpentSting).Ticking() and Target.Debuff(Aura.SerpentSting).Refreshable());
		end);
		-- actions.mb_ap_wfi_st+=/coordinated_assault
		action.EvaluateAction(Spell.CoordinatedAssault, true);
		-- actions.mb_ap_wfi_st+=/a_murder_of_crows
		action.EvaluateAction(Talent.AMurderOfCrows, true);
		-- actions.mb_ap_wfi_st+=/steel_trap
		action.EvaluateAction(Talent.SteelTrap, true);
		-- # To simulate usage for Mongoose Bite or Raptor Strike during Aspect of the Eagle, copy each occurrence of the action and append _eagle to the action name.
		-- We will do the next_wi_bomb by checking specifically if that spell object is in the players spell book using IsKnown.
		-- actions.mb_ap_wfi_st+=/mongoose_bite,if=buff.mongoose_fury.remains&next_wi_bomb.pheromone
		action.EvaluateAction(Talent.MongooseBite, function()
			return Player.Buff(Aura.MongooseFury).Up()
			   and Spell.PheromoneBomb.IsKnown();
		end);
		-- actions.mb_ap_wfi_st+=/kill_command,if=focus+cast_regen<focus.max&(buff.mongoose_fury.stack<5|focus<action.mongoose_bite.cost)
		action.EvaluateAction(Spell.KillCommand, function()
			return Player.Focus() + Player.Focus.CastRegen(Spell.KillCommand) < Player.Focus.Max()
			   and (Player.Buff(Aura.MongooseFury).Stack() < 5 or Player.Focus() < Talent.MongooseBite.Cost());
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions, this is the shrapnel bomb parts.
		-- We don't need to do the next_wi_bomb part, as the addon does this automatically as it won't suggest the spell if its not in the spellbook.
		-- actions.mb_ap_wfi_st+=/wildfire_bomb,if=next_wi_bomb.shrapnel&focus>60&dot.serpent_sting.remains>3*gcd
		action.EvaluateAction(Spell.ShrapnelBomb, function()
			return Player.Focus() > 60
			   and Target.Debuff(Aura.SerpentSting).Remains() > 3 * Player.GCD();
		end);
		-- We will do the next_wi_bomb by checking specifically if that spell object is in the players spell book using IsKnown.
		-- actions.mb_ap_wfi_st+=/serpent_sting,if=buff.vipers_venom.up|refreshable&(!talent.mongoose_bite.enabled|!talent.vipers_venom.enabled|next_wi_bomb.volatile&!dot.shrapnel_bomb.ticking|azerite.latent_poison.enabled|azerite.venomous_fangs.enabled)
		action.EvaluateAction(Spell.SerpentSting, function()
			return Player.Buff(Aura.VipersVenom).Up()
			    or Target.Debuff(Aura.SerpentSting).Refreshable()
			   and (not Talent.MongooseBite.Enabled() or not Talent.VipersVenom.Enabled() or Spell.VolatileBomb.IsKnown() and not Target.Debuff(Aura.ShrapnelBomb).Ticking() or Azerite.LatentPoison.Enabled() or Azerite.VenomousFangs.Enabled());
		end);
		-- actions.mb_ap_wfi_st+=/mongoose_bite,if=buff.mongoose_fury.up|focus>60|dot.shrapnel_bomb.ticking
		action.EvaluateAction(Talent.MongooseBite, function()
			return Player.Buff(Aura.MongooseFury).Up()
			    or Player.Focus() > 60
			    or Target.Debuff(Aura.ShrapnelBomb).Ticking();
		end);
		-- actions.mb_ap_wfi_st+=/serpent_sting,if=refreshable
		action.EvaluateAction(Spell.SerpentSting, function()
			return Target.Debuff(Aura.SerpentSting).Refreshable();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions so we need to split this line into 3 seperate actions.
		-- We don't need to do the next_wi_bomb part, as the addon does this automatically as it won't suggest the spell if its not in the spellbook.
		-- actions.mb_ap_wfi_st+=/wildfire_bomb,if=next_wi_bomb.volatile&dot.serpent_sting.ticking|next_wi_bomb.pheromone|next_wi_bomb.shrapnel&focus>50
		action.EvaluateAction(Spell.PheromoneBomb, true);
		action.EvaluateAction(Spell.ShrapnelBomb, function()
			return Player.Focus() > 50;
		end);
		action.EvaluateAction(Spell.VolatileBomb, function()
			return Target.Debuff(Aura.SerpentSting).Ticking();
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationMongooseBite = MongooseBite("MongooseBite");

-- SingleTarget Rotation
local function SingleTarget(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.st=a_murder_of_crows
		action.EvaluateAction(Talent.AMurderOfCrows, true);
		-- actions.st+=/coordinated_assault
		action.EvaluateAction(Spell.CoordinatedAssault, true);
		-- actions.st+=/wildfire_bomb,if=full_recharge_time<gcd&talent.alpha_predator.enabled
		action.EvaluateAction(Spell.WildfireBomb, function()
			return Spell.WildfireBomb.Charges.FullRechargeTime() < Player.GCD()
			   and Talent.AlphaPredator.Enabled();
		end);
		-- actions.st+=/serpent_sting,if=refreshable&buff.mongoose_fury.stack=5&talent.alpha_predator.enabled
		action.EvaluateAction(Spell.SerpentSting, function()
			return Player.Buff(Aura.MongooseFury).Stack() == 5
			   and Talent.AlphaPredator.Enabled();
		end);
		-- # To simulate usage for Mongoose Bite or Raptor Strike during Aspect of the Eagle, copy each occurrence of the action and append _eagle to the action name.
		-- actions.st+=/mongoose_bite,if=buff.mongoose_fury.stack=5&talent.alpha_predator.enabled
		action.EvaluateAction(Talent.MongooseBite, function()
			return Player.Buff(Aura.MongooseFury).Stack() == 5
			   and Talent.AlphaPredator.Enabled();
		end);
		-- actions.st+=/raptor_strike,if=talent.birds_of_prey.enabled&buff.coordinated_assault.up&(buff.coordinated_assault.remains<gcd|buff.blur_of_talons.up&buff.blur_of_talons.remains<gcd)
		action.EvaluateAction(Spell.RaptorStrike, function()
			return Talent.BirdsOfPrey.Enabled()
			   and Player.Buff(Aura.CoordinatedAssault).Up()
			   and (Player.Buff(Aura.CoordinatedAssault).Remains() < Player.GCD() or Player.Buff(Aura.BlurOfTalons).Up() and Player.Buff(Aura.BlurOfTalons).Remains() < Player.GCD());
		end);
		-- actions.st+=/mongoose_bite,if=talent.birds_of_prey.enabled&buff.coordinated_assault.up&(buff.coordinated_assault.remains<gcd|buff.blur_of_talons.up&buff.blur_of_talons.remains<gcd)
		action.EvaluateAction(Talent.MongooseBite, function()
			return Talent.BirdsOfPrey.Enabled()
			   and Player.Buff(Aura.CoordinatedAssault).Up()
			   and (Player.Buff(Aura.CoordinatedAssault).Remains() < Player.GCD() or Player.Buff(Aura.BlurOfTalons).Up() and Player.Buff(Aura.BlurOfTalons).Remains() < Player.GCD());
		end);
		-- actions.st+=/kill_command,if=focus+cast_regen<focus.max&buff.tip_of_the_spear.stack<3
		action.EvaluateAction(Spell.KillCommand, function()
			return Player.Focus() + Player.Focus.CastRegen(Spell.KillCommand) < Player.Focus.Max()
			   and Player.Buff(Aura.TipOfTheSpear).Stack() < 3;
		end);
		-- actions.st+=/chakrams
		action.EvaluateAction(Talent.Chakrams, true);
		-- actions.st+=/steel_trap
		action.EvaluateAction(Talent.SteelTrap, true);
		-- actions.st+=/wildfire_bomb,if=focus+cast_regen<focus.max&(full_recharge_time<gcd|dot.wildfire_bomb.refreshable&buff.mongoose_fury.down)
		action.EvaluateAction(Spell.WildfireBomb, function()
			return Player.Focus() + Player.Focus.CastRegen(Spell.WildfireBomb) < Player.Focus.Max()
			   and (Spell.WildfireBomb.Charges.FullRechargeTime() < Player.GCD() or Target.Debuff(Aura.WildfireBomb).Refreshable() and Player.Buff(Aura.MongooseFury).Down());
		end);
		-- actions.st+=/harpoon,if=talent.terms_of_engagement.enabled|azerite.up_close_and_personal.enabled
		action.EvaluateAction(Spell.Harpoon, function()
			return Talent.TermsOfEngagement.Enabled()
			    or Azerite.UpCloseAndPersonal.Enabled();
		end);
		-- actions.st+=/flanking_strike,if=focus+cast_regen<focus.max
		action.EvaluateAction(Talent.FlankingStrike, function()
			return Player.Focus() + Player.Focus.CastRegen(Talent.FlankingStrike) < Player.Focus.Max();
		end);
		-- actions.st+=/serpent_sting,if=buff.vipers_venom.up|refreshable&(!talent.mongoose_bite.enabled|!talent.vipers_venom.enabled|azerite.latent_poison.enabled|azerite.venomous_fangs.enabled)
		action.EvaluateAction(Spell.SerpentSting, function()
			return Player.Buff(Aura.VipersVenom).Up()
			    or Target.Debuff(Aura.SerpentSting).Refreshable()
			   and (not Talent.MongooseBite.Enabled() or not Talent.VipersVenom.Enabled() or Azerite.LatentPoison.Enabled() or Azerite.VenomousFangs.Enabled());
		end);
		-- actions.st+=/mongoose_bite,if=buff.mongoose_fury.up|focus>60
		action.EvaluateAction(Talent.MongooseBite, function()
			return Player.Buff(Aura.MongooseFury).Up()
			    or Player.Focus() > 60;
		end);
		-- actions.st+=/raptor_strike
		action.EvaluateAction(Spell.RaptorStrike, true);
		-- actions.st+=/wildfire_bomb,if=dot.wildfire_bomb.refreshable
		action.EvaluateAction(Spell.WildfireBomb, function()
			return Target.Debuff(Aura.WildfireBomb).Refreshable();
		end);
		-- actions.st+=/serpent_sting,if=refreshable
		action.EvaluateAction(Spell.SerpentSting, function()
			return Target.Debuff(Aura.SerpentSting).Refreshable();
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationSingleTarget = SingleTarget("SingleTarget");

-- WildfireInfusion Rotation
local function WildfireInfusion(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.wfi_st=a_murder_of_crows
		action.EvaluateAction(Talent.AMurderOfCrows, true);
		-- actions.wfi_st+=/coordinated_assault
		action.EvaluateAction(Spell.CoordinatedAssault, true);
		-- # To simulate usage for Mongoose Bite or Raptor Strike during Aspect of the Eagle, copy each occurrence of the action and append _eagle to the action name.
		-- We will do the next_wi_bomb by checking specifically if that spell object is in the players spell book using IsKnown.
		-- actions.wfi_st+=/mongoose_bite,if=azerite.wilderness_survival.enabled&next_wi_bomb.volatile&dot.serpent_sting.remains>2.1*gcd&dot.serpent_sting.remains<3.5*gcd&cooldown.wildfire_bomb.remains>2.5*gcd
		action.EvaluateAction(Talent.MongooseBite, function()
			return Azerite.WildernessSurvival.Enabled()
			   and Spell.VolatileBomb.IsKnown()
			   and Target.Debuff(Aura.SerpentSting).Remains() > 2.1 * Player.GCD()
			   and Target.Debuff(Aura.SerpentSting).Remains() < 3.5 * Player.GCD()
			   and (Spell.VolatileBomb.Cooldown.Remains() > 2.5 * Player.GCD() or coreSettings.GetRotationValue(Spell.WildfireBomb.CheckBox(), 2) ~= 2);
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions, this is the pheromone and volatile bomb parts.
		-- We don't need to do the next_wi_bomb part, as the addon does this automatically as it won't suggest the spell if its not in the spellbook.
		-- actions.wfi_st+=/wildfire_bomb,if=full_recharge_time<gcd|(focus+cast_regen<focus.max)&(next_wi_bomb.volatile&dot.serpent_sting.ticking&dot.serpent_sting.refreshable|next_wi_bomb.pheromone&!buff.mongoose_fury.up&focus+cast_regen<focus.max-action.kill_command.cast_regen*3)
		action.EvaluateAction(Spell.PheromoneBomb, function()
			return Spell.PheromoneBomb.Charges.FullRechargeTime() < Player.GCD()
			    or (Player.Focus() + Player.Focus.CastRegen(Spell.PheromoneBomb) < Player.Focus.Max())
			   and (not Player.Buff(Aura.MongooseFury).Up() and Player.Focus() + Player.Focus.CastRegen(Spell.PheromoneBomb) < Player.Focus.Max() - Player.Focus.CastRegen(Spell.KillCommand) * 3);
		end);
		action.EvaluateAction(Spell.VolatileBomb, function()
			return Spell.VolatileBomb.Charges.FullRechargeTime() < Player.GCD()
			    or (Player.Focus() + Player.Focus.CastRegen(Spell.VolatileBomb) < Player.Focus.Max())
			   and (Target.Debuff(Aura.SerpentSting).Ticking() and Target.Debuff(Aura.SerpentSting).Refreshable());
		end);
		-- actions.wfi_st+=/kill_command,if=focus+cast_regen<focus.max&buff.tip_of_the_spear.stack<3&(!talent.alpha_predator.enabled|buff.mongoose_fury.stack<5|focus<action.mongoose_bite.cost)
		action.EvaluateAction(Spell.KillCommand, function()
			return Player.Focus() + Player.Focus.CastRegen(Spell.KillCommand) < Player.Focus.Max()
			   and Player.Buff(Aura.TipOfTheSpear).Stack() < 3
			   and (not Talent.AlphaPredator.Enabled() or Player.Buff(Aura.MongooseFury).Stack() < 5 or Player.Focus() < Talent.MongooseBite.Cost());
		end);
		-- actions.wfi_st+=/raptor_strike,if=dot.internal_bleeding.stack<3&dot.shrapnel_bomb.ticking&!talent.mongoose_bite.enabled
		action.EvaluateAction(Spell.RaptorStrike, function()
			return Target.Debuff(Aura.InternalBleeding).Stack() < 3
			   and Target.Debuff(Aura.ShrapnelBomb).Ticking()
			   and not Talent.MongooseBite.Enabled();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions, this is the shrapnel bomb parts.
		-- We don't need to do the next_wi_bomb part, as the addon does this automatically as it won't suggest the spell if its not in the spellbook.
		-- actions.wfi_st+=/wildfire_bomb,if=next_wi_bomb.shrapnel&buff.mongoose_fury.down&(cooldown.kill_command.remains>gcd|focus>60)&!dot.serpent_sting.refreshable
		action.EvaluateAction(Spell.ShrapnelBomb, function()
			return Player.Buff(Aura.MongooseFury).Down()
			   and (Spell.KillCommand.Cooldown.Remains() > Player.GCD() or Player.Focus() > 60)
			   and not Target.Debuff(Aura.SerpentSting).Refreshable();
		end);
		-- actions.wfi_st+=/steel_trap
		action.EvaluateAction(Talent.SteelTrap, true);
		-- actions.wfi_st+=/flanking_strike,if=focus+cast_regen<focus.max
		action.EvaluateAction(Talent.FlankingStrike, function()
			return Player.Focus() + Player.Focus.CastRegen(Talent.FlankingStrike) < Player.Focus.Max();
		end);
		-- We will do the next_wi_bomb by checking specifically if that spell object is in the players spell book using IsKnown.
		-- actions.wfi_st+=/serpent_sting,if=buff.vipers_venom.up|refreshable&(!talent.mongoose_bite.enabled|!talent.vipers_venom.enabled|next_wi_bomb.volatile&!dot.shrapnel_bomb.ticking|azerite.latent_poison.enabled|azerite.venomous_fangs.enabled|buff.mongoose_fury.stack=5)
		action.EvaluateAction(Spell.SerpentSting, function()
			return Player.Buff(Aura.VipersVenom).Up()
			    or Target.Debuff(Aura.SerpentSting).Refreshable()
			   and (not Talent.MongooseBite.Enabled() or not Talent.VipersVenom.Enabled() or Spell.VolatileBomb.IsKnown() and not Target.Debuff(Aura.ShrapnelBomb).Ticking() or Azerite.LatentPoison.Enabled() or Azerite.VenomousFangs.Enabled() or Player.Buff(Aura.MongooseFury).Stack() == 5);
		end);
		-- actions.wfi_st+=/harpoon,if=talent.terms_of_engagement.enabled|azerite.up_close_and_personal.enabled
		action.EvaluateAction(Spell.Harpoon, function()
			return Talent.TermsOfEngagement.Enabled()
			    or Azerite.UpCloseAndPersonal.Enabled();
		end);
		-- actions.wfi_st+=/mongoose_bite,if=buff.mongoose_fury.up|focus>60|dot.shrapnel_bomb.ticking
		action.EvaluateAction(Talent.MongooseBite, function()
			return Player.Buff(Aura.MongooseFury).Up()
			    or Player.Focus() > 60
			    or Target.Debuff(Aura.ShrapnelBomb).Ticking();
		end);
		-- actions.wfi_st+=/raptor_strike
		action.EvaluateAction(Spell.RaptorStrike, true);
		-- actions.wfi_st+=/serpent_sting,if=refreshable
		action.EvaluateAction(Spell.SerpentSting, function()
			return Target.Debuff(Aura.SerpentSting).Refreshable();
		end);
		-- The addon has three seperate objects for Wildfire Bomb when using Wildfire Infusions so we need three copies of this line, one for each bomb type.
		-- We don't need to do the next_wi_bomb part, as the addon does this automatically as it won't suggest the spell if its not in the spellbook.
		-- actions.wfi_st+=/wildfire_bomb,if=next_wi_bomb.volatile&dot.serpent_sting.ticking|next_wi_bomb.pheromone|next_wi_bomb.shrapnel&focus>50
		action.EvaluateAction(Spell.PheromoneBomb, true);
		action.EvaluateAction(Spell.ShrapnelBomb, function()
			return Player.Focus() > 50;
		end);
		action.EvaluateAction(Spell.VolatileBomb, function()
			return Target.Debuff(Aura.SerpentSting).Ticking();
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationWildfireInfusion = WildfireInfusion("WildfireInfusion");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
	-- Inherits APL Class so get the base class.
	local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

	-- Store the information for the script.
	self.scriptInfo = {
		SpecializationID = self.SpecID,
		ScriptAuthor = "LunaEclipse",
		ScriptCredits = "HuntsTheWind",
		GuideAuthor = "Gray Hound and SimCraft",
		GuideLink = "http://www.wowhead.com/survival-hunter-guide",
		WoWVersion = 80001,
	};

	self.Defensives = {
		AspectOfTheTurtle = function()
			return Player.DamagePredicted(4) >= 40;
		end,

		Exhilaration = function()
			return Player.Health.Percent() < 70
			   and Player.DamagePredicted(5) >= 25;
		end,

		FeignDeath = function()
			return Player.Health.Percent() < 25
			   and Player.DamagePredicted(5) >= 25;
		end,

		MendPet = function()
			return not Pet.Buff(Aura.MendPet).Up()
			   and Pet.Health.Percent() < 80;
		end,
	};

	Objects.FinalizeRequirements(self.Defensives);

	-- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
	function self.Enable(...)
		Racial, Spell, Talent, Aura, Azerite = ...;

		Aura.BlurOfTalons = Objects.newSpell(277969);
		Aura.LatentPoison = Objects.newSpell(273286);

		Item = {};

		Consumable = {
			-- Potions
			BattlePotionOfAgility = Objects.newItem(163223),
		};

		Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for setting up the configuration screen, called when rotation becomes the active rotation.
	function self.SetupConfiguration(config, options)
		config.RacialOptions(options, Racial.AncestralCall, Racial.Berserking, Racial.BloodFury, Racial.Fireblood, Racial.LightsJudgment);
		config.AOEOptions(options, Spell.Carve, Spell.WildfireBomb);
		config.CooldownOptions(options, Talent.AMurderOfCrows, Talent.Butchery, Talent.Chakrams, Spell.CoordinatedAssault, Spell.KillCommand, Talent.MongooseBite, Spell.SerpentSting, Talent.SteelTrap);
		config.DefensiveOptions(options, Spell.AspectOfTheTurtle, Spell.Exhilaration, Spell.FeignDeath);
		config.PetOptions(options, Spell.CallPet, Spell.MendPet, Spell.RevivePet);
		config.UtilityOptions(options, Spell.Harpoon);
	end

	-- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
	function self.Disable()
		local coreTables = addon.Core.Tables;

		coreTables.Wipe(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for checking the rotation that displays on the Defensives icon.
	function self.Defensive(action)
		-- The abilities here should be listed from highest damage required to suggest to lowest,
		-- Specific damage types before all damage types.

		-- Heal pet if its losing health
		action.EvaluateDefensiveAction(Spell.MendPet, self.Defensives.MendPet);

		-- Protects against all types of damage
		action.EvaluateDefensiveAction(Spell.AspectOfTheTurtle, self.Defensives.AspectOfTheTurtle);

		-- Self Healing goes at the end and is only suggested if a major cooldown is not needed.
		action.EvaluateDefensiveAction(Spell.Exhilaration, self.Defensives.Exhilaration);

		-- Feign Death if no other defensives are available
		action.EvaluateDefensiveAction(Spell.FeignDeath, self.Defensives.FeignDeath);
	end

	-- Function for displaying opening rotation.
	function self.Opener(action)
	end

	-- Function for displaying any actions before combat starts.
	function self.Precombat(action)
		-- Not specified by SimCraft but ensure that pet is not already out before suggesting
		-- actions.precombat+=/summon_pet
		action.EvaluateAction(Spell.CallPet, function()
			return not Pet.IsActive();
		end);
		-- actions.precombat+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfAgility, true);
		-- actions.precombat+=/steel_trap
		action.EvaluateAction(Talent.SteelTrap, true);
		-- actions.precombat+=/harpoon
		action.EvaluateAction(Spell.Harpoon, true);
	end

	-- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
	function self.Combat(action)
		-- actions+=/call_action_list,name=cds
		action.CallActionList(rotationCooldowns);
		-- actions+=/run_action_list,name=mb_ap_wfi_st,if=active_enemies<2&talent.wildfire_infusion.enabled&talent.alpha_predator.enabled&talent.mongoose_bite.enabled
		action.RunActionList(rotationMongooseBite, function(numEnemies)
			return numEnemies < 2
			   and Talent.WildfireInfusion.Enabled()
			   and Talent.AlphaPredator.Enabled()
			   and Talent.MongooseBite.Enabled();
		end);
		-- actions+=/run_action_list,name=wfi_st,if=active_enemies<2&talent.wildfire_infusion.enabled
		action.RunActionList(rotationWildfireInfusion, function(numEnemies)
			return numEnemies < 2
			   and Talent.WildfireInfusion.Enabled();
		end);
		-- actions+=/run_action_list,name=st,if=active_enemies<2
		action.RunActionList(rotationSingleTarget, function(numEnemies)
			return numEnemies < 2;
		end);
		-- actions+=/run_action_list,name=cleave
		action.RunActionList(rotationCleave);
	end

	return self;
end

local rotationAPL = APL(nameAPL, "LunaEclipse: Survival Hunter", addonTable.Enum.SpecID.HUNTER_SURVIVAL);