local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

--- Localize Vars
local coreGeneral = addon.Core.General;
local coreSettings = addon.Core.Settings;
local apiGeneral = addon.API.General;
--local Enemies = addon.Modules.Enemies;
local Objects = addon.Core.Objects;
local val = coreGeneral.ToNumber;
local getVar = addonTable.RotationProcessing.GetValue;

-- Objects
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "lunaeclipse_mage_arcane";

-- Normal Rotation
local function Normal(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		ArcaneBarrage = {
			Burn = function(numEnemies)
				return getVar("burn_phase", true)
					and numEnemies >= 3
					and getVar("max_arcane_charges", true);
			end,

			Conserve = function()
				return getVar("conserve_phase", true)
					and Player.Buff(Aura.RuneOfPower).Down()
					and not Player.Casting.Spell(Talent.RuneOfPower)
					and Spell.ArcanePower.Cooldown.Remains() >= Spell.ArcaneBlast.ExecuteTime() * 4
					and Spell.Evocation.Cooldown.Remains() < 90 - 6
					and Spell.Evocation.Cooldown.Down()
					and (
								-- calculate if player has enough mana to spam AB until Evocation comes off CD, mana regen included
								-- Mana + Regen < AB quantity * AB cost
								-- if AB quantity = 10 and current arcane charge = 2 then total cost is: 1 AB with 2 charges, 1 AB with 3 charges, 8 AB with 4 charges
								Player.Mana.Current() + Player.Mana.Regen() * Spell.Evocation.Cooldown.Remains() < getVar("arcaneblast_base_cost", 0) * (((Spell.Evocation.Cooldown.Remains() / (Spell.ArcaneBlast.ExecuteTime() * (1 - (0.08 * Player.ArcaneCharges.Deficit()))) - Player.ArcaneCharges.Deficit()) * 5) + (Player.ArcaneCharges.Deficit() * (Player.ArcaneCharges.Deficit() + 1) / 2))
							or Spell.ArcanePower.Cooldown.Remains() <= Spell.Evocation.Cooldown.Remains() - 6
						)
					and (
								(
										getVar("max_arcane_charges", true)
									and (
												Player.Buff(Aura.RuleOfThrees).Down()
											or (
														Player.Buff(Aura.RuleOfThrees).Up()
													and Player.Casting.Spell(Spell.ArcaneBlast)
												)
										)
									and Player.Mana.Percent() <= getVar("conserve_mana", 0)
								)
							or (
										Player.Mana.Percent() <= getVar("conserve_mana", 0)
									and (
												not Talent.RuleOfThrees.Enabled()
											or Player.ArcaneCharges() == 2
											or (
														Player.ArcaneCharges == 1
													and Player.Casting.Spell(Spell.ArcaneBlast)
												)
										)
								)
							or (
										Spell.ArcanePower.Cooldown.Remains() > Talent.RuneOfPower.Charges.FullRechargeTime()
									and Player.Mana.Percent() <= getVar("conserve_mana", 0) + 25
									and (
												not Talent.RuleOfThrees.Enabled()
											or Player.ArcaneCharges() == 2
											or (
														Player.ArcaneCharges == 1
													and Player.Casting.Spell(Spell.ArcaneBlast)
												)
										)
								)
							or (
										Talent.ArcaneOrb.Enabled()
									and Talent.ArcaneOrb.Cooldown.Remains() <= Player.GCD()
									and Spell.ArcanePower.Cooldown.Remains() > 10
								)
						    or Player.Mana.Percent() <= getVar("conserve_mana", 0) - 10
					    );
			end,

			Abort = function()
				return not (coreSettings.GetRotationValue(Spell.ArcanePower.CheckBox(), 2) == 2)
					and Spell.ArcanePower.Cooldown.Up()
					and getVar("conserve_phase", true)
					and (
								(
										getVar("max_arcane_charges", true)
									and (
												Player.Buff(Aura.RuleOfThrees).Down()
											or (
														Player.Buff(Aura.RuleOfThrees).Up()
													and Player.Casting.Spell(Spell.ArcaneBlast)
												)
										)
									and Player.Mana.Percent() <= getVar("conserve_mana", 0)
								)
							or (
										Player.Mana.Percent() <= getVar("conserve_mana", 0)
									and (
												not Talent.RuleOfThrees.Enabled()
											or Player.ArcaneCharges() == 2
											or (
														Player.ArcaneCharges == 1
													and Player.Casting.Spell(Spell.ArcaneBlast)
												)
										)
								)
						    or Player.Mana.Percent() <= getVar("conserve_mana", 0) - 10
				    	);
			end,
		},

		ArcaneBlast = {
			RuleOfThrees = function()
				return (
								Player.Buff(Aura.RuleOfThrees).Up()
							and not Player.Casting.Spell(Spell.ArcaneBlast)
						)
				   	or (
					   			Player.ArcaneCharges() == 2
					   		and Player.Casting.Spell(Spell.ArcaneBlast)
					   		and Talent.RuleOfThrees.Enabled()
				   		);
			end,

			Burn = function()
				return getVar("burn_phase", true)
			end,

			Conserve = function()
				return getVar("conserve_phase", true)
					and (
								Player.ArcaneCharges.Deficit() >= 0
							or (
										Player.ArcaneCharges.Deficit() >= 1
									and Player.Casting.Spell(Spell.ArcaneBlast)
								)
						);
			end,
		},

		ArcaneExplosion = {
			Burn = function(numEnemies)
				return getVar("burn_phase", true)
					and numEnemies >= 3;
			end,

			Conserve = function(numEnemies)
				return getVar("conserve_phase", true)
					and numEnemies >= 3
				   	and (
					   			Player.Mana.Percent() >= getVar("conserve_mana", 0)
					   		or Player.ArcaneCharges() == 2
					   		or (
					   					Player.ArcaneCharges() == 1
					   				and Player.Casting.Spell(Spell.ArcaneBlast)
					   			)
				   		);
			end,
		},

		ArcaneFamiliar = function()
			return not Player.Buff(Aura.ArcaneFamiliar).Up()
				or Player.Buff(Aura.ArcaneFamiliar).Remains() < 15 * 60;
		end,

		ArcaneIntellect = function()
			return not Player.Buff(Aura.ArcaneIntellect).Up()
				or Player.Buff(Aura.ArcaneIntellect).Remains() < 15 * 60;
		end,

		ArcaneMissiles = {
			Burn = function(numEnemies)
				return getVar("burn_phase", true)
					and Player.Buff(Aura.Clearcasting).Up()
				   	and numEnemies < 3
				   	and (
				   				Talent.Amplification.Enabled()
				   			or (
				   						not Talent.Overpowered.Enabled()
				   					and Azerite.ArcanePummeling.Rank() >= 2
				   				)
				   			or Player.Buff(Aura.ArcanePower).Down()
				   		);
			end,

			Conserve = function(numEnemies)
				return getVar("conserve_phase", true)
					and Player.Mana.Percent() <= 95
					and numEnemies < 3
					and (
								Spell.Evocation.Cooldown.Remains() < 90 - 6
							or Spell.Evocation.Cooldown.Up()
						)
				   	and Player.Buff(Aura.Clearcasting).Up();
			end,
		},
		
		ArcaneOrb = {
			Burn = function(numEnemies)
				return getVar("burn_phase", true)
					and (
								Player.ArcaneCharges() == 0
						    or (
						    			numEnemies < 3
						    		or (
						    					numEnemies < 2
						    				and Talent.Resonance.Enabled()
						    			)
						    	)
						);
			end,

			Conserve = function(numEnemies)
				return getVar("conserve_phase", true)
					and (
							(
									Player.ArcaneCharges() <= 2
								or (
											Player.ArcaneCharges() <= 1
										and Player.Casting.Spell(Spell.ArcaneBlast)
									)
							)
						   	and (
						   				Spell.ArcanePower.Cooldown.Remains() > 10
						   			or numEnemies <= 2
						   		)
						);
			end,
		},

		ArcanePower = function()
			return getVar("max_arcane_charges", true)
				and (
							not Talent.RuneOfPower.Enabled()
						or Player.Casting.Spell(Talent.RuneOfPower)
						or Player.Buff(Aura.RuneOfPower).Remains() >= 6
					);
		end,

		BattlePotionOfIntellect = function()
			return Player.Buff(Aura.ArcanePower).Up()
			   and (
			   				Player.Buff(Aura.Berserking).Up()
			   			or Player.Buff(Aura.BloodFury).Up()
			   			or not (
			   							Player.Race("Troll")
			   						or Player.Race("Orc")
			   					)
			   		);
		end,

		ChargedUp = function()
			return Player.ArcaneCharges() == 0;
		end,

		Evocation = function()
			return Player.Mana.Percent() >= 85;
		end,

		LightsJudgment = function()
			return Player.Buff(Aura.ArcanePower).Down();
		end,

		MirrorImage = function()
			return getVar("conserve_phase", true)
				and (
							Spell.ArcanePower.Cooldown.Remains() <= Player.GCD()
						or Spell.ArcanePower.Cooldown.Remains() > Talent.MirrorImage.Cooldown.Base() / 3
					);
		end,

		NetherTempest = function()
			return Target.Debuff(Aura.NetherTempest).Refreshable()
			   and getVar("max_arcane_charges", true)
			   and Player.Buff(Aura.RuneOfPower).Down()
			   and Player.Buff(Aura.ArcanePower).Down();
		end,

		PresenceOfMind = function()
			return Player.Buff(Aura.RuneOfPower).Remains() >= 2 * Player.GCD()
			    or Player.Buff(Aura.ArcanePower).Remains() >= 2 * Player.GCD();
		end,

		RuneOfPower = function()
			return getVar("max_arcane_charges", true)
				and Talent.RuneOfPower.Cooldown.Up()
				and not Player.Casting.Spell(Talent.RuneOfPower)
				and not Player.Buff(Aura.ArcanePower).Up()
				and Player.Buff(Aura.RuneOfPower).Remains() < Player.GCD()
			    and (
			    			Spell.ArcanePower.Cooldown.Remains() <= Talent.RuneOfPower.ExecuteTime()
			    		or (
			    					Player.Mana.Percent() >= getVar("conserve_mana", 0)
			    				and (
								   			Talent.RuneOfPower.Charges.FullRechargeTime() <= Talent.RuneOfPower.ExecuteTime()
								   		or Talent.RuneOfPower.Charges.FullRechargeTime() <= Spell.ArcanePower.Cooldown.Remains()
								   		or Target.TimeToDie() <= Spell.ArcanePower.Cooldown.Remains()
								   	)
			    			)
			   		);
		end,

		Supernova = function()
			return getVar("conserve_phase", true)
				and Player.Mana.Percent() <= 80
				and (
							Spell.Evocation.Cooldown.Remains() < 90 - 6
						or Spell.Evocation.Cooldown.Up()
					);
		end,
	};

	Objects.FinalizeRequirements(self.Requirements);

	function self.Rotation(action)
		action.EvaluateAction(Spell.ArcaneIntellect, self.Requirements.ArcaneIntellect);
		action.EvaluateAction(Talent.ArcaneFamiliar, self.Requirements.ArcaneFamiliar);
		action.EvaluateAction(Talent.MirrorImage, self.Requirements.MirrorImage);
		action.EvaluateAction(Talent.ChargedUp, self.Requirements.ChargedUp);
		action.EvaluateAction(Talent.ArcaneOrb, self.Requirements.ArcaneOrb.Burn);
		action.EvaluateAction(Talent.ArcaneOrb, self.Requirements.ArcaneOrb.Conserve);
		action.EvaluateAction(Talent.NetherTempest, self.Requirements.NetherTempest);
		action.EvaluateAction(Racial.LightsJudgment, self.Requirements.LightsJudgment);
		action.EvaluateAction(Talent.RuneOfPower, self.Requirements.RuneOfPower);
		action.EvaluateAction(Spell.ArcaneBlast, self.Requirements.ArcaneBlast.RuleOfThrees);
		action.EvaluateAction(Spell.ArcanePower, self.Requirements.ArcanePower);
		action.EvaluateAction(Racial.BloodFury, true);
		action.EvaluateAction(Racial.Berserking, true);
		action.EvaluateAction(Racial.Fireblood, true);
		action.EvaluateAction(Racial.AncestralCall, true);
		action.EvaluateAction(Spell.PresenceOfMind, self.Requirements.PresenceOfMind);
		action.EvaluateAction(Consumable.BattlePotionOfIntellect, self.Requirements.BattlePotionOfIntellect);
		action.EvaluateAction(Spell.ArcaneBarrage, self.Requirements.ArcaneBarrage.Abort);
		action.EvaluateAction(Spell.ArcaneBarrage, self.Requirements.ArcaneBarrage.Burn);
		action.EvaluateAction(Spell.ArcaneExplosion, self.Requirements.ArcaneExplosion.Burn);
		action.EvaluateInterruptCondition(Spell.ArcaneMissiles, self.Requirements.ArcaneMissiles.Burn, true);
		action.EvaluateAction(Spell.ArcaneBlast, self.Requirements.ArcaneBlast.Burn);
		action.EvaluateAction(Talent.Supernova, self.Requirements.Supernova);
		action.EvaluateInterruptCondition(Spell.ArcaneMissiles, self.Requirements.ArcaneMissiles.Conserve, true);
		action.EvaluateAction(Spell.ArcaneBarrage, self.Requirements.ArcaneBarrage.Conserve);
		action.EvaluateAction(Spell.ArcaneExplosion, self.Requirements.ArcaneExplosion.Conserve);
		action.EvaluateAction(Spell.ArcaneBlast, self.Requirements.ArcaneBlast.Conserve);
		action.EvaluateInterruptCondition(Spell.Evocation, true, self.Requirements.Evocation);
		action.EvaluateAction(Spell.ArcaneBarrage, true);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationNormal = Normal("Normal");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
	-- Inherits APL Class so get the base class.
	local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

	-- Store the information for the script.
	self.scriptInfo = {
		SpecializationID = self.SpecID,
		ScriptAuthor = "LunaEclipse",
		GuideAuthor = "Malon and SimCraft",
		GuideLink = "https://www.wowhead.com/arcane-mage-simple-guide",
		WoWVersion = 80001,
	};

	-- Table to hold requirements to use spells for defensive reasons.
	self.Defensives = {
		-- Greater invisibility will only be used when solo if you have use stealth while solo checked, because it will drop you out of combat.
		GreaterInvisibility = function()
			return (
							not Player.IsSolo()
						or coreSettings.GetCharacterValue("ScriptOptions", "OPT_STEALTH_SOLO") == 1
					)
			   and Player.DamagePredicted(5) >= 40;
		end,

		IceBlock = function()
			return Player.DamagePredicted(5) >= 60;
		end,

		PrismaticBarrier = function()
			return not Player.Buff(Aura.PrismaticBarrier).Up()
			   and Player.DamagePredicted(5) >= 15;
		end,
	};

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		ArcaneBlast = function()
			return Player.Buff(Aura.RuleOfThrees).Up();
		end,

		-- SimCraft doesn't specify this, but only cast the buff if you don't have it.
		-- actions.precombat+=/summon_arcane_familiar
		ArcaneFamiliar = function()
			return not Player.Buff(Aura.ArcaneFamiliar).Up()
				or Player.Buff(Aura.ArcaneFamiliar).Remains() < 15 * 60;
		end,

		-- SimCraft doesn't specify this, but only cast the buff if you don't have it.
		-- actions.precombat+=/arcane_intellect
		ArcaneIntellect = function()
			return not Player.Buff(Aura.ArcaneIntellect).Up()
				or Player.Buff(Aura.ArcaneIntellect).Remains() < 15 * 60;
		end,

		ArcanePower = function()
			return getVar("max_arcane_charges", true)
				and (
							not Talent.RuneOfPower.Enabled()
						or Player.Casting.Spell(Talent.RuneOfPower)
						or Player.Buff(Aura.RuneOfPower).Remains() >= 6
					);
		end,

		ChargedUp = function()
			return Player.ArcaneCharges() == 0;
		end,

		RuneOfPower = function()
			return getVar("max_arcane_charges", true)
				and not Player.Casting.Spell(Talent.RuneOfPower)
			    and (
				   			Talent.RuneOfPower.Charges.FullRechargeTime() <= Talent.RuneOfPower.ExecuteTime() + Player.GCD()
				   		or Talent.RuneOfPower.Charges.FullRechargeTime() <= Spell.ArcanePower.Cooldown.Remains() + Player.GCD()
				   		or Target.TimeToDie() <= Spell.ArcanePower.Cooldown.Remains()
			   		);
		end,
	};

	Objects.FinalizeRequirements(self.Defensives, self.Requirements);

	-- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
	function self.Enable(...)
		Racial, Spell, Talent, Aura, Azerite = ...;

		Aura.DivineRight = Objects.newSpell(278523);

		Item = {};

		Consumable = {
			-- Potions
			BattlePotionOfIntellect = Objects.newItem(163222),
		};

		Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for setting up the configuration screen, called when rotation becomes the active rotation.
	function self.SetupConfiguration(config, options)
		config.RacialOptions(options, Racial.AncestralCall, Racial.Berserking, Racial.BloodFury, Racial.Fireblood, Racial.LightsJudgment);
		config.AOEOptions(options, Spell.ArcaneExplosion, Talent.ArcaneOrb, Talent.NetherTempest, Talent.Supernova);
		config.BuffOptions(options, Talent.ArcaneFamiliar, Spell.ArcaneIntellect);
		config.CooldownOptions(options, Spell.ArcanePower, Talent.ChargedUp, Spell.Evocation, Talent.MirrorImage, Spell.PresenceOfMind, Talent.RuneOfPower);
		config.DefensiveOptions(options, Spell.GreaterInvisibility, Spell.IceBlock, Spell.PrismaticBarrier);
	end

	-- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
	function self.Disable()
		local coreTables = addon.Core.Tables;

		coreTables.Wipe(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for checking the rotation that displays on the Defensives icon.
	function self.Defensive(action)
		-- The abilities here should be listed from highest damage required to suggest to lowest,
		-- Specific damage types before all damage types.

		-- Protects against all types of damage
		action.EvaluateDefensiveAction(Spell.IceBlock, self.Defensives.IceBlock);
		action.EvaluateDefensiveAction(Spell.GreaterInvisibility, self.Defensives.GreaterInvisibility);
		action.EvaluateDefensiveAction(Spell.PrismaticBarrier, self.Defensives.PrismaticBarrier);
	end

	-- Function for displaying opening rotation.
	function self.Opener(action)
	end

	-- Function for displaying any actions before combat starts.
	function self.Precombat(action)
		action.SetValue("conserve_mana", function() return 60 - 20 * val(Talent.Overpowered.Enabled()) + 10 * val(Talent.RuneOfPower.Enabled()) end);
		action.SetValue("arcaneblast_base_cost", function()	return Spell.ArcaneBlast.Cost() / (Player.ArcaneCharges() + 1) end);
		action.SetValue("max_arcane_charges", function() return Player.ArcaneCharges() == Player.ArcaneCharges.Max() or (Player.ArcaneCharges() == 3 and Player.Casting.Spell(Spell.ArcaneBlast)) end);

		action.EvaluateAction(Spell.ArcaneIntellect, self.Requirements.ArcaneIntellect);
		action.EvaluateAction(Talent.ArcaneFamiliar, self.Requirements.ArcaneFamiliar);
		action.EvaluateAction(Talent.ChargedUp, self.Requirements.ChargedUp);
		-- actions.precombat+=/mirror_image
		action.EvaluateAction(Talent.MirrorImage, true);
		-- actions.precombat+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfIntellect, true);
		action.EvaluateAction(Talent.RuneOfPower, self.Requirements.RuneOfPower);
		action.EvaluateAction(Spell.ArcaneBlast, self.Requirements.ArcaneBlast);
		action.EvaluateAction(Spell.ArcanePower, self.Requirements.ArcanePower);
		-- actions.precombat+=/arcane_blast
		action.EvaluateAction(Spell.ArcaneBlast, true);
	end

	-- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
	function self.Combat(action)
		action.SetValue("burn_phase", function() return Player.Buff(Aura.ArcanePower).Remains() >= Spell.ArcaneBlast.ExecuteTime() or Player.Buff(Aura.RuneOfPower).Remains() >= Spell.ArcaneBlast.ExecuteTime() end);
		action.SetValue("conserve_phase", function() return Player.Buff(Aura.ArcanePower).Remains() < Spell.ArcaneBlast.ExecuteTime() and Player.Buff(Aura.RuneOfPower).Remains() < Spell.ArcaneBlast.ExecuteTime() end);
		action.SetValue("max_arcane_charges", function() return Player.ArcaneCharges() == Player.ArcaneCharges.Max() or (Player.ArcaneCharges() == 3 and Player.Casting.Spell(Spell.ArcaneBlast)) end);

		action.CallActionList(rotationNormal);
	end

	return self;
end

local rotationAPL = APL(nameAPL, "LunaEclipse: Arcane Mage", addonTable.Enum.SpecID.MAGE_ARCANE);
