local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

--- Localize Vars
local coreGeneral = addon.Core.General;
local Objects = addon.Core.Objects;
local Settings = addon.Core.Settings;
local val = coreGeneral.ToNumber;

-- Objects
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "lunaeclipse_mage_fire";

local function spell_haste()
	return 1 + (addon.Core.Misc.GetHaste() / 100);
end

-- ActiveTalents Rotation
local function ActiveTalents(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		-- actions.active_talents=blast_wave,if=(buff.combustion.down)|(buff.combustion.up&action.fire_blast.charges<1)
		BlastWave = function()
			return Player.Buff(Aura.Combustion).Down()
			    or (Player.Buff(Aura.Combustion).Up() and Spell.FireBlast.Charges() < 1);
		end,

		-- actions.active_talents+=/dragons_breath,if=talent.alexstraszas_fury.enabled&!buff.hot_streak.react
		DragonsBreath = function()
			return Talent.AlexstraszasFury.Enabled()
			   and not Player.Buff(Aura.HotStreak).Up();
		end,

		-- actions.active_talents+=/living_bomb,if=active_enemies>1&buff.combustion.down
		LivingBomb = function(numEnemies)
			return numEnemies > 1
			   and Player.Buff(Aura.Combustion).Down();
		end,

		-- Firestarter is active when you have Firestarter talent and the target is above 90% health
		-- actions.active_talents+=/meteor,if=cooldown.combustion.remains>40|(cooldown.combustion.remains>target.time_to_die)|buff.rune_of_power.up|firestarter.active
		Meteor = function()
			return Spell.Combustion.Cooldown.Remains() > 40
			    or (Spell.Combustion.Cooldown.Remains() > Target.TimeToDie())
			    or Player.Buff(Aura.RuneOfPower).Up()
			    or (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90);
		end,
	};

	Objects.FinalizeRequirements(self.Requirements);

	function self.Rotation(action)
		action.EvaluateAction(Talent.BlastWave, self.Requirements.BlastWave);
		action.EvaluateAction(Talent.Meteor, self.Requirements.Meteor);
		action.EvaluateAction(Spell.DragonsBreath, self.Requirements.DragonsBreath);
		action.EvaluateAction(Talent.LivingBomb, self.Requirements.LivingBomb);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationActiveTalents = ActiveTalents("ActiveTalents");

-- Combustion Rotation
local function Combustion(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		-- actions.combustion_phase+=/dragons_breath,if=!buff.hot_streak.react&action.fire_blast.charges<1
		DragonsBreath = function()
			return not Player.Buff(Aura.HotStreak).Up()
			   and Spell.FireBlast.Charges() < 1;
		end,

		-- actions.combustion_phase+=/fire_blast,if=buff.heating_up.react
		FireBlast = function()
			return Player.Buff(Aura.HeatingUp).Up();
		end,

		-- actions.combustion_phase+=/flamestrike,if=((talent.flame_patch.enabled&active_enemies>2)|active_enemies>6)&buff.hot_streak.react
		Flamestrike = function(numEnemies)
			return ((Talent.FlamePatch.Enabled() and numEnemies > 2) or numEnemies > 6)
			   and Player.Buff(Aura.HotStreak).Up()
		end,

		-- actions.combustion_phase=lights_judgment,if=buff.combustion.down
		LightsJudgment = function()
			return Player.Buff(Aura.Combustion).Down();
		end,

		Pyroblast = {
			-- actions.combustion_phase+=/pyroblast,if=buff.pyroclasm.react&buff.combustion.remains>execute_time
			Pyroclasm = function()
				return Player.Buff(Aura.Pyroclasm).Up()
				   and Player.Buff(Aura.Combustion).Remains() > Spell.Pyroblast.ExecuteTime();
			end,

			-- actions.combustion_phase+=/pyroblast,if=buff.hot_streak.react
			Use = function()
				return Player.Buff(Aura.HotStreak).Up();
			end,
		},

		-- actions.combustion_phase+=/rune_of_power,if=buff.combustion.down
		RuneOfPower = function()
			return Player.Buff(Aura.Combustion).Down();
		end,

		Scorch = {
			-- actions.combustion_phase+=/scorch,if=target.health.pct<=30&talent.searing_touch.enabled
			SearingTouch = function()
				return Target.Health.Percent() <= 30
				   and Talent.SearingTouch.Enabled();
			end,

			-- actions.combustion_phase+=/scorch,if=buff.combustion.remains>cast_time
			Use = function()
				return Player.Buff(Aura.Combustion).Remains() > Spell.Scorch.CastTime();
			end,
		},
	};

	Objects.FinalizeRequirements(self.Requirements);

	function self.Rotation(action)
		action.EvaluateAction(Racial.LightsJudgment, self.Requirements.LightsJudgment);
		action.EvaluateAction(Talent.RuneOfPower, self.Requirements.RuneOfPower);

		-- actions.combustion_phase+=/call_action_list,name=active_talents
		action.CallActionList(rotationActiveTalents);

		-- actions.combustion_phase+=/combustion
		action.EvaluateAction(Spell.Combustion, true);
		-- actions.combustion_phase+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfIntellect, true);
		-- actions.combustion_phase+=/blood_fury
		action.EvaluateAction(Racial.BloodFury, true);
		-- actions.combustion_phase+=/berserking
		action.EvaluateAction(Racial.Berserking, true);
		-- actions.combustion_phase+=/fireblood
		action.EvaluateAction(Racial.Fireblood, true);
		-- actions.combustion_phase+=/ancestral_call
		action.EvaluateAction(Racial.AncestralCall, true);
		action.EvaluateAction(Spell.Flamestrike, self.Requirements.Flamestrike);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.Pyroclasm);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.Use);
		action.EvaluateAction(Spell.FireBlast, self.Requirements.FireBlast);
		-- actions.combustion_phase+=/phoenix_flames
		action.EvaluateAction(Talent.PhoenixFlames, true);
		action.EvaluateAction(Spell.Scorch, self.Requirements.Scorch.Use);
		action.EvaluateAction(Spell.DragonsBreath, self.Requirements.DragonsBreath);
		action.EvaluateAction(Spell.Scorch, self.Requirements.Scorch.SearingTouch);
	end

	-- Firestarter is active when you have Firestarter talent and the target is above 90% health
	-- actions+=/call_action_list,name=combustion_phase,if=cooldown.combustion.remains<=action.rune_of_power.cast_time+(!talent.kindling.enabled*gcd)&(!talent.firestarter.enabled|!firestarter.active|active_enemies>=4|active_enemies>=2&talent.flame_patch.enabled)|buff.combustion.up
	function self.Use(numEnemies)
		return Spell.Combustion.Cooldown.Remains() <= Talent.RuneOfPower.CastTime() + (val(not Talent.Kindling.Enabled()) * Player.GCD())
		   and (not Talent.Firestarter.Enabled() or not (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90) or numEnemies >= 4 or numEnemies >= 2 and Talent.FlamePatch.Enabled())
		    or Player.Buff(Aura.Combustion).Up();
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationCombustion = Combustion("Combustion");

-- RuneOfPower Rotation
local function RuneOfPower(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		-- actions.rop_phase+=/dragons_breath,if=active_enemies>2
		DragonsBreath = function(numEnemies)
			return numEnemies > 2;
		end,

		FireBlast = {
			-- Firestarter is active when you have Firestarter talent and the target is above 90% health
			-- actions.rop_phase+=/fire_blast,if=!prev_off_gcd.fire_blast&buff.heating_up.react&firestarter.active&charges_fractional>1.7
			Firestarter = function()
				return not Player.PrevOffGCD(1, Spell.FireBlast)
				   and Player.Buff(Aura.HeatingUp).Up()
				   and (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90)
				   and Spell.FireBlast.Charges.Fractional() > 1.7;
			end,

			-- Firestarter is active when you have Firestarter talent and the target is above 90% health
			-- actions.rop_phase+=/fire_blast,if=!prev_off_gcd.fire_blast&!firestarter.active
			Use = function()
				return not Player.PrevOffGCD(1, Spell.FireBlast)
				   and not (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90);
			end,
		},

		Flamestrike = {
			-- actions.rop_phase+=/flamestrike,if=((talent.flame_patch.enabled&active_enemies>1)|active_enemies>4)&buff.hot_streak.react
			HotStreak = function(numEnemies)
				return ((Talent.FlamePatch.Enabled() and numEnemies > 1) or numEnemies > 4)
				   and Player.Buff(Aura.HotStreak).Up();
			end,

			-- actions.rop_phase+=/flamestrike,if=(talent.flame_patch.enabled&active_enemies>2)|active_enemies>5
			Use = function(numEnemies)
				return (Talent.FlamePatch.Enabled() and numEnemies > 2)
				    or numEnemies > 5;
			end,
		},

		PhoenixFlames = {
			-- Firestarter is active when you have Firestarter talent and the target is above 90% health
			-- actions.rop_phase+=/phoenix_flames,if=!prev_gcd.1.phoenix_flames&charges_fractional>2.7&firestarter.active
			Firestarter = function()
				return not Player.PrevGCD(1, Talent.PhoenixFlames)
				   and Talent.PhoenixFlames.Charges.Fractional() > 2.7
				   and (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90);
			end,

			-- actions.rop_phase+=/phoenix_flames,if=!prev_gcd.1.phoenix_flames
			Use = function()
				return not Player.PrevGCD(1, Talent.PhoenixFlames);
			end,
		},

		Pyroblast = {
			-- actions.rop_phase+=/pyroblast,if=buff.hot_streak.react
			HotStreak = function()
				return Player.Buff(Aura.HotStreak).Up();
			end,

			-- actions.rop_phase+=/pyroblast,if=buff.pyroclasm.react&execute_time<buff.pyroclasm.remains&buff.rune_of_power.remains>cast_time
			Use = function()
				return Player.Buff(Aura.Pyroclasm).Up()
				   and Spell.Pyroblast.ExecuteTime() < Player.Buff(Aura.Pyroclasm).Remains()
				   and Player.Buff(Aura.RuneOfPower).Remains() > Spell.Pyroblast.CastTime();
			end,
		},

		-- actions.rop_phase+=/scorch,if=target.health.pct<=30&talent.searing_touch.enabled
		Scorch = function()
			return Target.Health.Percent() <= 30
			   and Talent.SearingTouch.Enabled();
		end,
	};

	Objects.FinalizeRequirements(self.Requirements);

	function self.Rotation(action)
		-- actions.rop_phase=rune_of_power
		action.EvaluateAction(Talent.RuneOfPower, true);
		action.EvaluateAction(Spell.Flamestrike, self.Requirements.Flamestrike.HotStreak);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.HotStreak);

		-- actions.rop_phase+=/call_action_list,name=active_talents
		action.CallActionList(rotationActiveTalents);

		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.Use);
		action.EvaluateAction(Spell.FireBlast, self.Requirements.FireBlast.Firestarter);
		action.EvaluateAction(Talent.PhoenixFlames, self.Requirements.PhoenixFlames.Firestarter);
		action.EvaluateAction(Spell.FireBlast, self.Requirements.FireBlast.Use);
		action.EvaluateAction(Talent.PhoenixFlames, self.Requirements.PhoenixFlames.Use);
		action.EvaluateAction(Spell.Scorch, self.Requirements.Scorch);
		action.EvaluateAction(Spell.DragonsBreath, self.Requirements.DragonsBreath);
		action.EvaluateAction(Spell.Flamestrike, self.Requirements.Flamestrike.Use);
		-- actions.rop_phase+=/fireball
		action.EvaluateAction(Spell.Fireball, true);
	end

	-- actions+=/call_action_list,name=rop_phase,if=buff.rune_of_power.up&buff.combustion.down
	function self.Use()
		return Player.Buff(Aura.RuneOfPower).Up()
		   and Player.Buff(Aura.Combustion).Down();
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationRuneOfPower = RuneOfPower("RuneOfPower");

-- Standard Rotation
local function Standard(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		FireBlast = {
			-- actions.standard_rotation+=/fire_blast,if=talent.kindling.enabled&buff.heating_up.react&(!talent.rune_of_power.enabled|charges_fractional>1.5|cooldown.combustion.remains<40)&(3-charges_fractional)*(18*spell_haste)<cooldown.combustion.remains+3|target.time_to_die<4
			Kindling = function()
				return Talent.Kindling.Enabled()
				   and Player.Buff(Aura.HeatingUp).Up()
				   and (not Talent.RuneOfPower.Enabled() or Spell.FireBlast.Charges.Fractional() > 1.5 or Spell.Combustion.Cooldown.Remains() < 40)
				   and (3 - Spell.FireBlast.Charges.Fractional()) * (18 * spell_haste()) < Spell.Combustion.Cooldown.Remains() + 3
				    or Target.TimeToDie() < 4;
			end,

			-- actions.standard_rotation+=/fire_blast,if=!talent.kindling.enabled&buff.heating_up.react&(!talent.rune_of_power.enabled|charges_fractional>1.4|cooldown.combustion.remains<40)&(3-charges_fractional)*(12*spell_haste)<cooldown.combustion.remains+3|target.time_to_die<4
			Use = function()
				return not Talent.Kindling.Enabled()
				   and Player.Buff(Aura.HeatingUp).Up()
				   and (not Talent.RuneOfPower.Enabled() or Spell.FireBlast.Charges.Fractional() > 1.4 or Spell.Combustion.Cooldown.Remains() < 40)
				   and (3 - Spell.FireBlast.Charges.Fractional()) * (12 * spell_haste()) < Spell.Combustion.Cooldown.Remains() + 3
				    or Target.TimeToDie() < 4;
			end,
		},

		-- actions.standard_rotation=flamestrike,if=((talent.flame_patch.enabled&active_enemies>1)|active_enemies>4)&buff.hot_streak.react
		Flamestrike = function(numEnemies)
			return ((Talent.FlamePatch.Enabled() and numEnemies > 1) or numEnemies > 4)
			   and Player.Buff(Aura.HotStreak).Up();
		end,

		PhoenixFlames = {
			-- actions.standard_rotation+=/phoenix_flames,if=charges_fractional>2.7&active_enemies>2
			Charges = function(numEnemies)
				return Talent.PhoenixFlames.Charges.Fractional() > 2.7
				   and numEnemies > 2;
			end,

			-- actions.standard_rotation+=/phoenix_flames,if=(buff.combustion.up|buff.rune_of_power.up)&(4-charges_fractional)*30<cooldown.combustion.remains+5
			LongCooldown = function()
				return (Player.Buff(Aura.Combustion).Up() or Player.Buff(Aura.RuneOfPower).Up())
				   and (4 - Talent.PhoenixFlames.Charges.Fractional()) * 30 < Spell.Combustion.Cooldown.Remains() + 5;
			end,

			-- actions.standard_rotation+=/phoenix_flames,if=(buff.combustion.up|buff.rune_of_power.up|buff.incanters_flow.stack>3|talent.mirror_image.enabled)&(4-charges_fractional)*13<cooldown.combustion.remains+5|target.time_to_die<10
			ShortCooldown = function()
				return (Player.Buff(Aura.Combustion).Up() or Player.Buff(Aura.RuneOfPower).Up() or Player.Buff(Aura.IncantersFlow).Stack() > 3 or Talent.MirrorImage.Enabled())
				   and (4 - Talent.PhoenixFlames.Charges.Fractional()) * 13 < Spell.Combustion.Cooldown.Remains() + 5
				    or Target.TimeToDie() < 10;
			end,

			-- actions.standard_rotation+=/phoenix_flames,if=charges_fractional>2.5&cooldown.combustion.remains>23
			Use = function()
				return Talent.PhoenixFlames.Charges.Fractional() > 2.5
				   and Spell.Combustion.Cooldown.Remains() > 23;
			end,
		},

		Pyroblast = {
			-- Firestarter is active when you have Firestarter talent and the target is above 90% health
			-- actions.standard_rotation+=/pyroblast,if=buff.hot_streak.react&firestarter.active&!talent.rune_of_power.enabled
			Forestarter = function()
				return Player.Buff(Aura.HotStreak).Up()
				   and (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90)
				   and not Talent.RuneOfPower.Enabled();
			end,

			-- actions.standard_rotation+=/pyroblast,if=buff.hot_streak.react&buff.hot_streak.remains<action.fireball.execute_time
			HotStreak = function()
				return Player.Buff(Aura.HotStreak).Up()
				   and Player.Buff(Aura.HotStreak).Remains() < Spell.Fireball.ExecuteTime();
			end,

			-- actions.standard_rotation+=/pyroblast,if=buff.pyroclasm.react&execute_time<buff.pyroclasm.remains
			Pyroclasm = function()
				return Player.Buff(Aura.Pyroclasm).Up()
				   and Spell.Pyroblast.ExecuteTime() < Player.Buff(Aura.Pyroclasm).Remains();
			end,

			-- actions.standard_rotation+=/pyroblast,if=buff.hot_streak.react&target.health.pct<=30&talent.searing_touch.enabled
			SearingTouch = function()
				return Player.Buff(Aura.HotStreak).Up()
				   and Target.Health.Percent() <= 30
				   and Talent.SearingTouch.Enabled();
			end,

			-- actions.standard_rotation+=/pyroblast,if=buff.hot_streak.react&(!prev_gcd.1.pyroblast|action.pyroblast.in_flight)
			Use = function()
				return Player.Buff(Aura.HotStreak).Up()
				   and (not Player.PrevGCD(1, Spell.Pyroblast) or Spell.Pyroblast.IsInFlight());
			end,
		},

		-- actions.standard_rotation+=/scorch,if=(target.health.pct<=30&talent.searing_touch.enabled)|(azerite.preheat.enabled&debuff.preheat.down)
		Scorch = function()
			return (Target.Health.Percent() <= 30 and Talent.SearingTouch.Enabled())
			    or (Azerite.Preheat.Enabled() and Target.Debuff(Aura.Preheat).Down());
		end,
	};

	Objects.FinalizeRequirements(self.Requirements);

	function self.Rotation(action)
		action.EvaluateAction(Spell.Flamestrike, self.Requirements.Flamestrike);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.HotStreak);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.Firestarter);
		action.EvaluateAction(Talent.PhoenixFlames, self.Requirements.PhoenixFlames.Charges);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.Use);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.SearingTouch);
		action.EvaluateAction(Spell.Pyroblast, self.Requirements.Pyroblast.Pyroclasm);

		-- actions.standard_rotation+=/call_action_list,name=active_talents
		action.CallActionList(rotationActiveTalents);

		action.EvaluateAction(Spell.FireBlast, self.Requirements.FireBlast.Use);
		action.EvaluateAction(Spell.FireBlast, self.Requirements.FireBlast.Kindling);
		action.EvaluateAction(Talent.PhoenixFlames, self.Requirements.PhoenixFlames.ShortCooldown);
		action.EvaluateAction(Talent.PhoenixFlames, self.Requirements.PhoenixFlames.LongCooldown);
		action.EvaluateAction(Talent.PhoenixFlames, self.Requirements.PhoenixFlames.Use);
		action.EvaluateAction(Spell.Scorch, self.Requirements.Scorch);
		-- actions.standard_rotation+=/fireball
		action.EvaluateAction(Spell.Fireball, true);
		-- actions.standard_rotation+=/scorch
		action.EvaluateAction(Spell.Scorch, true);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationStandard = Standard("Standard");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
	-- Inherits APL Class so get the base class.
	local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

	-- Store the information for the script.
	self.scriptInfo = {
		SpecializationID = self.SpecID,
		ScriptAuthor = "LunaEclipse",
		GuideAuthor = "Malon and SimCraft",
		GuideLink = "https://www.wowhead.com/fire-mage-simple-guide",
		WoWVersion = 80001,
	};

	-- Table to hold requirements to use spells for defensive reasons.
	self.Defensives = {
		BlazingBarrier = function()
			return not Player.Buff(Aura.BlazingBarrier).Up()
			   and Player.DamagePredicted(5) >= 15;
		end,

		IceBlock = function()
			return Player.DamagePredicted(5) >= 60;
		end,

		-- Greater invisibility will only be used when solo if you have use stealth while solo checked, because it will drop you out of combat.
		Invisibility = function()
			return (not Player.IsSolo() or Settings.GetCharacterValue("ScriptOptions", "OPT_STEALTH_SOLO") == 1)
			   and Player.DamagePredicted(5) >= 40;
		end,
	};

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {
		-- SimCraft doesn't specify this, but only cast the buff if you don't have it.
		-- actions.precombat+=/arcane_intellect
		ArcaneIntellect = function()
			return not Player.Buff(Aura.ArcaneIntellect).Up();
		end,

		-- actions+=/mirror_image,if=buff.combustion.down
		MirrorImage = function()
			return Player.Buff(Aura.Combustion).Down();
		end,

		RuneOfPower = {
			-- # RoP use while using Pyroclasm.
			-- actions+=/rune_of_power,if=buff.pyroclasm.react&(cooldown.combustion.remains>40|action.rune_of_power.charges>1)
			Pyroclasm = function()
				return Player.Buff(Aura.Pyroclasm).Up()
				   and (Spell.Combustion.Cooldown.Remains() > 40 or Talent.RuneOfPower.Charges() > 1);
			end,

			-- # Standard Talent RoP Logic.
			-- Firestarter is active when you have Firestarter talent and the target is above 90% health
			-- actions+=/rune_of_power,if=firestarter.active&action.rune_of_power.charges=2|cooldown.combustion.remains>40&buff.combustion.down&!talent.kindling.enabled|target.time_to_die<11|talent.kindling.enabled&(charges_fractional>1.8|time<40)&cooldown.combustion.remains>40
			Use = function()
				return (Talent.Firestarter.Enabled() and Target.Health.Percent() > 90)
				   and Talent.RuneOfPower.Charges() == 2
				    or Spell.Combustion.Cooldown.Remains() > 40
				   and Player.Buff(Aura.Combustion).Down()
				   and not Talent.Kindling.Enabled()
				    or Target.TimeToDie() < 11
				    or Talent.Kindling.Enabled() and (Talent.RuneOfPower.Charges.Fractional() > 1.8 or coreGeneral.CombatTime() < 40)
				   and Spell.Combustion.Cooldown.Remains() > 40;
			end,
		},
	};

	Objects.FinalizeRequirements(self.Defensives, self.Requirements);

	-- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
	function self.Enable(...)
		Racial, Spell, Talent, Aura, Azerite = ...;

		Aura.Preheat = Objects.newSpell(273333);

		Item = {};

		Consumable = {
			-- Potions
			BattlePotionOfIntellect = Objects.newItem(163222),
		};

		Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for setting up the configuration screen, called when rotation becomes the active rotation.
	function self.SetupConfiguration(config, options)
		config.RacialOptions(options, Racial.AncestralCall, Racial.Berserking, Racial.BloodFury, Racial.Fireblood, Racial.LightsJudgment);
		config.AOEOptions(options, Talent.BlastWave, Spell.DragonsBreath, Spell.Flamestrike, Talent.LivingBomb, Talent.Meteor, Talent.PhoenixFlames);
		config.BuffOptions(options, Spell.ArcaneIntellect);
		config.CooldownOptions(options, Spell.Combustion, Talent.MirrorImage, Talent.RuneOfPower);
		config.DefensiveOptions(options, Spell.BlazingBarrier, Spell.IceBlock, Spell.Invisibility);
	end

	-- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
	function self.Disable()
		local coreTables = addon.Core.Tables;

		coreTables.Wipe(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for checking the rotation that displays on the Defensives icon.
	function self.Defensive(action)
		-- The abilities here should be listed from highest damage required to suggest to lowest,
		-- Specific damage types before all damage types.

		-- Protects against all types of damage
		action.EvaluateDefensiveAction(Spell.IceBlock, self.Defensives.IceBlock);
		action.EvaluateDefensiveAction(Spell.Invisibility, self.Defensives.Invisibility);
		action.EvaluateDefensiveAction(Spell.BlazingBarrier, self.Defensives.BlazingBarrier);
	end

	-- Function for displaying opening rotation.
	function self.Opener(action)
	end

	-- Function for displaying any actions before combat starts.
	function self.Precombat(action)
		action.EvaluateAction(Spell.ArcaneIntellect, self.Requirements.ArcaneIntellect);
		-- actions.precombat+=/mirror_image
		action.EvaluateAction(Talent.MirrorImage, true);
		-- actions.precombat+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfIntellect, true);
		-- actions.precombat+=/pyroblast
		action.EvaluateAction(Spell.Pyroblast, true);
	end

	-- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
	function self.Combat(action)
		action.EvaluateAction(Talent.MirrorImage, self.Requirements.MirrorImage);
		action.EvaluateAction(Talent.RuneOfPower, self.Requirements.RuneOfPower.Use);
		action.EvaluateAction(Talent.RuneOfPower, self.Requirements.RuneOfPower.Pyroclasm);

		action.CallActionList(rotationCombustion);
		action.CallActionList(rotationRuneOfPower);
		-- actions+=/call_action_list,name=standard_rotation
		action.CallActionList(rotationStandard);
	end

	return self;
end

local rotationAPL = APL(nameAPL, "LunaEclipse: Fire Mage", addonTable.Enum.SpecID.MAGE_FIRE);
