local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

--- Localize Vars
local Objects = addon.Core.Objects;

-- Objects
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "huntsthewind_monk_windwalker";

-- CD Rotation
local function Cooldowns(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.cd+=/arcane_torrent,if=chi.max-chi>=1&energy.time_to_max>=0.5
        ArcaneTorrent = function()
            return Player.Chi.Deficit() >= 1
                    and Player.Energy.TimeToMax() >= 0.5;
        end,

        -- actions.cd+=/serenity,if=cooldown.rising_sun_kick.remains<=2|target.time_to_die<=12
        Serenity = function()
            return Spell.RisingSunKick.Cooldown.Remains() <= 2
                or Target.TimeToDie() <= 12;
        end,

        -- actions.cd+=/storm_earth_and_fire,if=cooldown.storm_earth_and_fire.charges=2|(cooldown.fists_of_fury.remains<=6&chi>=3&cooldown.rising_sun_kick.remains<=1)|target.time_to_die<=15
        StormEarthAndFire = function()
            return Spell.StormEarthAndFire.Charges() == 2
                or (Spell.FistsOfFury.Cooldown.Remains() <= 6
                    and Player.Chi() >= 3
                    and Spell.RisingSunKick.Cooldown.Remains() <= 1)
                or Target.TimeToDie() <= 15;
        end,

        -- actions.cd+=/touch_of_death,if=target.time_to_die>9
        TouchOfDeath = function()
            return Target.TimeToDie() > 9;
        end,

    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        -- actions.cd=invoke_xuen_the_white_tiger
        action.EvaluateAction(Talent.InvokeXuenTheWhiteTiger, true);
        -- actions.cd+=/blood_fury
        action.EvaluateAction(Racial.BloodFury, true);
        -- actions.cd+=/berserking
        action.EvaluateAction(Racial.Berserking, true);
        action.EvaluateAction(Racial.ArcaneTorrent, self.Requirements.ArcaneTorrent);
        -- actions.cd+=/lights_judgment
        action.EvaluateAction(Racial.LightsJudgment, true);
        action.EvaluateAction(Racial.Fireblood, true);
        action.EvaluateAction(Racial.AncestralCall, true);
        action.EvaluateAction(Spell.TouchOfDeath, self.Requirements.TouchOfDeath);
        action.EvaluateAction(Spell.StormEarthAndFire, self.Requirements.StormEarthAndFire);
        action.EvaluateAction(Talent.Serenity, self.Requirements.Serenity);
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationCooldowns = Cooldowns("Cooldowns");

-- AOE Rotation
local function AOE(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.aoe+=/blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&(buff.bok_proc.up|(talent.hit_combo.enabled&prev_gcd.1.tiger_palm&chi<4))
        BlackoutKick = function()
            return not Player.PrevGCD(1, Spell.BlackoutKick)
                and (Player.Buff(Aura.BlackoutKick).Up()
                    or (Talent.HitCombo.Enabled()
                        and Player.PrevGCD(1, Spell.TigerPalm())
                        and Player.Chi() < 4));
        end,

        -- actions.aoe+=/chi_burst,if=chi<=3
        ChiBurst = function()
            return Player.Chi() <= 3;
        end,

        -- actions.aoe+=/energizing_elixir,if=!prev_gcd.1.tiger_palm&chi<=1&energy<50
        EnergizingElixir = function()
            return not Player.PrevGCD(1, Spell.TigerPalm)
                and Player.Chi() <= 1
                and Player.Energy() < 50;
        end,

        -- actions.aoe+=/fists_of_fury,if=energy.time_to_max>3
        FistsOfFury = function()
            return Player.Energy.TimeToMax() > 3;
        end,

        -- actions.aoe+=/fist_of_the_white_tiger,if=chi.max-chi>=3&(energy>46|buff.rushing_jade_wind.down)
        FistOfTheWhiteTiger = function()
            return Player.Chi.Deficit() >= 3
                and (Player.Energy() > 46
                    or Player.Buff(Aura.RushingJadeWind).Down());
        end,

        -- actions.aoe+=/flying_serpent_kick,if=buff.bok_proc.down,interrupt=1
        FlyingSerpentKick = function()
            return Player.Buff(Aura.BlackoutKick).Down();
        end,

        -- actions.aoe+=/rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(talent.whirling_dragon_punch.enabled&cooldown.whirling_dragon_punch.remains<5)&cooldown.fists_of_fury.remains>3
        RisingSunKick = function()
            return Talent.WhirlingDragonPunch.Enabled()
                and Talent.WhirlingDragonPunch.Cooldown.Remains() < 5
                and Spell.FistsOfFury.Cooldown.Remains() > 3;
        end,

        -- actions.aoe+=/rushing_jade_wind,if=buff.rushing_jade_wind.down&energy.time_to_max>1
        RushingJadeWind = function()
            return Player.Buff(Aura.RushingJadeWind).Down()
                and Player.Energy.TimeToMax() > 1;
        end,

        -- actions.aoe+=/spinning_crane_kick,if=!prev_gcd.1.spinning_crane_kick&(chi>3|cooldown.fists_of_fury.remains>6)&(chi>=5|cooldown.fists_of_fury.remains>2)
        -- actions.aoe+=/spinning_crane_kick,if=!prev_gcd.1.spinning_crane_kick&(((chi>3|cooldown.fists_of_fury.remains>6)&(chi>=5|cooldown.fists_of_fury.remains>2))|energy.time_to_max<=3)
        SpinningCraneKick = function()
            return not Player.PrevGCD(1, Spell.SpinningCraneKick)
                and (((Player.Chi() > 3
                        or Spell.FistsOfFury.Cooldown.Remains() > 6)
                    and (Player.Chi() >= 5
                        or Spell.FistsOfFury.Cooldown.Remains() > 2))
                    or Player.Energy.TimeToMax() <= 3);
        end,

        -- actions.aoe+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=chi.max-chi>=2&(energy>56|buff.rushing_jade_wind.down)&(!talent.hit_combo.enabled|!prev_gcd.1.tiger_palm)
        TigerPalm = function()
            return Player.Chi.Deficit() >= 2
                and (Player.Energy() > 56
                    or Player.Buff(Aura.RushingJadeWind).Down())
                and (not Talent.HitCombo.Enabled()
                    or not Player.PrevGCD(1, Spell.TigerPalm));
        end,

    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        -- actions.aoe+=/whirling_dragon_punch
        action.EvaluateAction(Talent.WhirlingDragonPunch, true);
        action.EvaluateAction(Talent.EnergizingElixir, self.Requirements.EnergizingElixir);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury);
        action.EvaluateAction(Talent.RushingJadeWind, self.Requirements.RushingJadeWind);
        action.EvaluateCycleAction(Spell.RisingSunKick, self.Requirements.RisingSunKick);
        action.EvaluateAction(Spell.SpinningCraneKick, self.Requirements.SpinningCraneKick);
        action.EvaluateAction(Talent.ChiBurst, self.Requirements.ChiBurst);
        action.EvaluateAction(Talent.FistOfTheWhiteTiger, self.Requirements.FistOfTheWhiteTiger);
        action.EvaluateCycleAction(Spell.TigerPalm, self.Requirements.TigerPalm);
        -- actions.aoe+=/chi_wave
        action.EvaluateAction(Talent.ChiWave, true);
        action.EvaluateCycleAction(Spell.BlackoutKick, self.Requirements.BlackoutKick);
    end

    function self.Use(numEnemies)
        return numEnemies >= 3
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationAOE = AOE("AOE");

-- Serenity Rotation
local function Serenity(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.serenity+=/fists_of_fury,if=(buff.bloodlust.up&prev_gcd.1.rising_sun_kick&!azerite.swift_roundhouse.enabled)|buff.serenity.remains<1|(active_enemies>1&active_enemies<5)
        FistsOfFury = function(numEnemies)
            return (Player.HasBloodlust()
                    and Player.PrevGCD(1, Spell.RisingSunKick)
                    and not Azerite.SwiftRoundhouse.Enabled())
                or Player.Buff(Aura.Serenity).Remains() < 1
                or (numEnemies > 1
                    and numEnemies < 5);
        end,

        -- actions.serenity=rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies<3|prev_gcd.1.spinning_crane_kick
        RisingSunKick = function(numEnemies)
                return numEnemies < 3
                    or Player.PrevGCD(1, Spell.SpinningCraneKick);
            end,

        -- actions.serenity+=/spinning_crane_kick,if=!prev_gcd.1.spinning_crane_kick&(active_enemies>=3|(active_enemies=2&prev_gcd.1.blackout_kick))
        SpinningCraneKick = function(numEnemies)
            return not Player.PrevGCD(1, Spell.SpinningCraneKick)
                and (numEnemies >= 3
                    or (numEnemies == 2
                        and Player.PrevGCD(1, Spell.BlackoutKick)));
        end,

    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        action.EvaluateCycleAction(Spell.RisingSunKick, self.Requirements.RisingSunKick);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury);
        action.EvaluateAction(Spell.SpinningCraneKick, self.Requirements.SpinningCraneKick);
        action.EvaluateCycleAction(Spell.BlackoutKick, true);
    end

    -- actions+=/call_action_list,name=serenity,if=buff.serenity.up
    function self.Use()
        return Player.Buff(Aura.Serenity).Up();
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationSerenity = Serenity("Serenity");

-- Single Target Rotation
local function SingleTarget(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.st+=/blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&(cooldown.rising_sun_kick.remains>3|chi>=3)&(cooldown.fists_of_fury.remains>4|chi>=4|(chi=2&prev_gcd.1.tiger_palm))&buff.swift_roundhouse.stack<2
        BlackoutKick = function()
            return not Player.PrevGCD(1, Spell.BlackoutKick)
                and (Spell.RisingSunKick.Cooldown.Remains() > 3
                    or Player.Chi() >= 3)
                and (Spell.FistsOfFury.Cooldown.Remains() > 4
                    or Player.Chi() >= 4
                    or (Player.Chi() == 2
                        and Player.PrevGCD(1, Spell.TigerPalm)))
                and Player.Buff(Aura.SwiftRoundhouse).Stack() < 2;
        end,

        -- actions.st+=/chi_burst,if=chi.max-chi>=1&active_enemies=1|chi.max-chi>=2
        ChiBurst = function(numEnemies)
            return (Player.Chi.Deficit() >= 1
                    and numEnemies == 1)
                or Player.Chi.Deficit() >= 2;
        end,

        -- actions.st+=/energizing_elixir,if=chi<=3&energy<50
        EnergizingElixir = function()
            return Player.Chi() <= 3
                and Player.Energy() < 50;
        end,

        -- actions.st+=/fists_of_fury,if=energy.time_to_max>3
        FistsOfFury = function()
            return Player.Energy.TimeToMax() > 3;
        end,

        -- actions.st+=/fist_of_the_white_tiger,if=chi<=2&(buff.rushing_jade_wind.down|energy>46)
        FistOfTheWhiteTiger = function()
            return Player.Chi() <= 2
                and (Player.Buff(Aura.RushingJadeWind).Down()
                    or Player.Energy() > 46);
        end,

        -- actions.st+=/flying_serpent_kick,if=prev_gcd.1.blackout_kick&chi>3&buff.swift_roundhouse.stack<2,interrupt=1
        FlyingSerpentKick = function()
            return Player.PrevGCD(1, Spell.BlackoutKick)
                and Player.Chi() > 3
                and Player.Buff(Aura.SwiftRoundhouse).Stack() < 2;
        end,

        RisingSunKick = {
            -- actions.st+=/rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=chi>=5
            HighChi = function()
                return Player.Chi() >= 5;
            end,

            -- actions.st+=/rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains
            Use = function()
                return true;
            end,

        },

        RushingJadeWind = {
            -- actions.st+=/rushing_jade_wind,if=buff.rushing_jade_wind.down&energy.time_to_max>1&active_enemies>1
            Activate = function(numEnemies)
                return Player.Buff(Aura.RushingJadeWind).Down()
                    and Player.Energy.TimeToMax() > 1
                    and numEnemies > 1;
            end,

            -- actions.st=cancel_buff,name=rushing_jade_wind,if=active_enemies=1&(!talent.serenity.enabled|cooldown.serenity.remains>3)
            Cancel = function(numEnemies)
                return numEnemies == 1
                    and (not Talent.Serenity.Enabled()
                        or Talent.Serenity.Cooldown.Remains() > 3);
            end,

        },

        -- actions.st+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&chi.max-chi>=2&(buff.rushing_jade_wind.down|energy>56)
        TigerPalm = function()
            return not Player.PrevGCD(1, Spell.TigerPalm)
                and Player.Chi.Deficit() >= 2
                and (Player.Buff(Aura.RushingJadeWind).Down()
                    or Player.Energy() > 56);
        end,

    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        action.EvaluateAction(Talent.RushingJadeWind, self.Requirements.RushingJadeWind.Cancel);
        -- actions.st+=/whirling_dragon_punch
        action.EvaluateAction(Talent.WhirlingDragonPunch, true);
        action.EvaluateCycleAction(Spell.RisingSunKick, self.Requirements.RisingSunKick.HighChi);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury);
        action.EvaluateCycleAction(Spell.RisingSunKick, self.Requirements.RisingSunKick.Use);
        action.EvaluateAction(Talent.RushingJadeWind, self.Requirements.RushingJadeWind.Activate);
        action.EvaluateAction(Talent.FistOfTheWhiteTiger, self.Requirements.FistOfTheWhiteTiger);
        action.EvaluateAction(Talent.EnergizingElixir, self.Requirements.EnergizingElixir);
        action.EvaluateCycleAction(Spell.BlackoutKick, self.Requirements.BlackoutKick);
        -- actions.st+=/chi_wave
        action.EvaluateAction(Talent.ChiWave, true);
        action.EvaluateAction(Talent.ChiBurst, self.Requirements.ChiBurst);
        action.EvaluateCycleAction(Spell.TigerPalm, self.Requirements.TigerPalm);
    end

    function self.Use(numEnemies)
        return numEnemies < 3;
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationSingleTarget = SingleTarget("SingleTarget");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
    -- Inherits APL Class so get the base class.
    local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

    -- Store the information for the script.
    self.scriptInfo = {
        SpecializationID = addonTable.MONK_WINDWALKER,
        ScriptAuthor = "HuntsTheWind",
        GuideAuthor = "Babylonius and SimCraft 76c224",
        GuideLink = "http://www.icy-veins.com/wow/windwalker-monk-pve-dps-guide",
        WoWVersion = 80001,
    };

    self.Defensives = {
        DampenHarm = function()
            return Player.PhysicalDamagePredicted(5) >= 25;
        end,

        DiffuseMagic = function()
            return Player.MagicDamagePredicted(3) >= 30;
        end,

        TouchOfKarma = function()
            return Player.DamagePredicted(5) >= 25;
        end,
    };

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- trinket.proc.agility.react can't be done without long lists of hard coded trinket data, so just skip trinket.proc conditions.
        -- actions+=/potion,if=buff.serenity.up|buff.storm_earth_and_fire.up|(!talent.serenity.enabled&trinket.proc.agility.react)|buff.bloodlust.react|target.time_to_die<=60
        BurstingBlood = function()
            return Player.Buff(Aura.Serenity).Up()
                or Player.Buff(Aura.StormEarthAndFire).Up()
                or not Talent.Serenity.Enabled()
                or Player.HasBloodlust()
                or Target.TimeToDie() <= 60;
        end,

        -- actions.precombat+=/chi_burst,if=(!talent.serenity.enabled|!talent.fist_of_the_white_tiger.enabled)
        ChiBurst = function()
            return not Talent.Serenity.Enabled()
                or not Talent.FistOfTheWhiteTiger.Enabled();
        end,

        -- actions+=/fist_of_the_white_tiger,if=(energy.time_to_max<1|(talent.serenity.enabled&cooldown.serenity.remains<2))&chi.max-chi>=3
        FistOfTheWhiteTiger = function()
            return (Player.Energy.TimeToMax() < 1
                    or (Talent.Serenity.Enabled()
                        and Talent.Serenity.Cooldown.Remains() < 2))
                and Player.Chi.Deficit() >= 3;
        end,

        -- actions+=/rushing_jade_wind,if=talent.serenity.enabled&cooldown.serenity.remains<3&energy.time_to_max>1&buff.rushing_jade_wind.down
        RushingJadeWind = function()
            return Talent.Serenity.Enabled()
                and Talent.Serenity.Cooldown.Remains() < 3
                and Player.Energy.TimeToMax() > 1
                and Player.Buff(Aura.RushingJadeWind).Down();
        end,

        -- not in simcraft. This is a condition to keep Rising Sun Kick from
        -- getting "stuck" in the rotation as a long cooldown, due to the very
        -- restrictive conditions on Blackout Kick and Tiger Palm for mastery
        TigerPalm =  {
            Unclog = function()
                return Player.PrevGCD(1, Spell.TigerPalm)
                    and Spell.RisingSunKick.Cooldown.Remains() > 3;
            end,

            -- actions+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=(energy.time_to_max<1|(talent.serenity.enabled&cooldown.serenity.remains<2))&chi.max-chi>=2&!prev_gcd.1.tiger_palm
            Use = function()
                return (Player.Energy.TimeToMax() < 1
                        or (Talent.Serenity.Enabled()
                        and Talent.Serenity.Cooldown.Remains() < 2))
                        and Player.Chi.Deficit() >= 2
                        and not Player.PrevGCD(1, Spell.TigerPalm);
            end,

        },
        -- NOTE: need to understand this simcraft language better
        -- actions+=/touch_of_karma,interval=90,pct_health=0.5,if=!talent.good_karma.enabled,interval=90,pct_health=0.5
        -- actions+=/touch_of_karma,interval=90,pct_health=1,if=talent.good_karma.enabled,interval=90,pct_health=1
        TouchOfKarma = function()
            return true;
        end,

    };

    Objects.FinalizeRequirements(self.Defensives, self.Requirements);

    -- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
    function self.Enable(...)
        -- Spells
        Racial, Spell, Talent, Aura, Azerite = ...;

        Spell.CancelFlyingSerpentKick = Objects.newSpell(115057);

        Aura.SwiftRoundhouse = Objects.newSpell(277669);

        Item = {};

        Consumable = {
            -- Potions
            BurstingBlood = Objects.newItem(152560),
        };

        Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
    end

    -- Function for setting up the configuration screen, called when rotation becomes the active rotation.
    function self.SetupConfiguration(config, options)
        config.RacialOptions(options, Racial.ArcaneTorrent, Racial.Berserking, Racial.BloodFury, Racial.GiftOfTheNaaru, Racial.Shadowmeld);
        config.AOEOptions(options, Spell.FistsOfFury, Talent.RushingJadeWind, Spell.SpinningCraneKick, Talent.ChiBurst);
        config.CooldownOptions(options, Talent.ChiWave, Talent.EnergizingElixir, Spell.FlyingSerpentKick, Talent.InvokeXuenTheWhiteTiger, Talent.Serenity, Spell.StormEarthAndFire, Spell.TouchOfDeath, Talent.WhirlingDragonPunch);
        config.DefensiveOptions(options, Talent.DampenHarm, Talent.DiffuseMagic, Spell.TouchOfKarma);
    end

    -- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
    function self.Disable()
        local coreTables = addon.Core.Tables;

        coreTables.Wipe(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
    end

    -- Function for checking the rotation that displays on the Defensives icon.
    function self.Defensive(action)
        -- The abilities here should be listed from highest damage required to suggest to lowest,
        -- Specific damage types before all damage types.

        -- Redirect damage back to target
        action.EvaluateDefensiveAction(Spell.TouchOfKarma, self.Defensives.TouchOfKarma);

        -- Protects against magical damage
        action.EvaluateDefensiveAction(Talent.DiffuseMagic, self.Defensives.DiffuseMagic);

        -- Protects against physical damage
        action.EvaluateDefensiveAction(Talent.DampenHarm, self.Defensives.DampenHarm);
    end

    -- Function for displaying opening rotation.
    function self.Opener(action)
    end

    -- Function for displaying any actions before combat starts.
    function self.Precombat(action)
        -- actions.precombat+=/potion
        action.EvaluateAction(Consumable.BurstingBlood, true);
        -- actions.precombat+=/chi_burst
        action.EvaluateAction(Talent.ChiBurst, self.Requirements.ChiBurst);
        -- actions.precombat+=/chi_wave
        action.EvaluateAction(Talent.ChiWave, true);
        -- in case player has chosen Eye of the Tiger at lvl 15
        action.EvaluateAction(Spell.TouchOfDeath, true, Target.InRange(5));
        -- in case other spells are on cooldown and target is in melee range
        action.EvaluateAction(Spell.TigerPalm, true, Target.InRange(5));
        -- in case other spells are on cooldown and target out of melee range
        action.EvaluateAction(Spell.CracklingJadeLightning, true);
    end

    -- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
    function self.Combat(action)
        action.EvaluateAction(Talent.RushingJadeWind, self.Requirements.RushingJadeWind);
        action.EvaluateAction(Spell.TouchOfKarma, self.Requirements.TouchOfKarma);
        action.EvaluateAction(Consumable.BurstingBlood, self.Requirements.BurstingBlood);
        action.CallActionList(rotationSerenity);
        action.EvaluateAction(Talent.FistOfTheWhiteTiger, self.Requirements.FistOfTheWhiteTiger);
        action.EvaluateCycleAction(Spell.TigerPalm, self.Requirements.TigerPalm.Use);
        action.CallActionList(rotationCooldowns);
        action.CallActionList(rotationSingleTarget);
        action.CallActionList(rotationAOE);
        -- test of extra TP action to prevent getting stuck on RSK
        action.EvaluateAction(Spell.TigerPalm, self.Requirements.TigerPalm.Unclog);
    end

    return self;
end

local rotationAPL = APL(nameAPL, "HuntsTheWind: Windwalker Monk", addonTable.Enum.SpecID.MONK_WINDWALKER);
