local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];

--- Localize Vars
local coreGeneral = addon.Core.General;
local Enemies = addon.Modules.Enemies;
local Objects = addon.Core.Objects;

-- Objects
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "lunaeclipse_warrior_arms";

-- AOE Rotation
local function AOE(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.five_target=skullsplitter,if=rage<60&(!talent.deadly_calm.enabled|buff.deadly_calm.down)
		action.EvaluateAction(Talent.Skullsplitter, function()
			return Player.Rage() < 60
			   and (not Talent.DeadlyCalm.Enabled() or Player.Buff(Aura.DeadlyCalm).Down());
		end);
		-- actions.five_target+=/ravager,if=(!talent.warbreaker.enabled|cooldown.warbreaker.remains<2)
		action.EvaluateAction(Talent.Ravager, function()
			return not Talent.Warbreaker.Enabled()
			    or Talent.Warbreaker.Cooldown.Remains() < 2;
		end);
		-- actions.five_target+=/colossus_smash,if=debuff.colossus_smash.down
		action.EvaluateAction(Spell.ColossusSmash, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.five_target+=/warbreaker,if=debuff.colossus_smash.down
		action.EvaluateAction(Talent.Warbreaker, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.five_target+=/bladestorm,if=buff.sweeping_strikes.down&(!talent.deadly_calm.enabled|buff.deadly_calm.down)&((debuff.colossus_smash.remains>4.5&!azerite.test_of_might.enabled)|buff.test_of_might.up)
		action.EvaluateAction(Spell.Bladestorm, function()
			return Player.Buff(Aura.SweepingStrikes).Down()
			   and (not Talent.DeadlyCalm.Enabled() or Player.Buff(Aura.DeadlyCalm).Down())
			   and ((Target.Debuff(Aura.ColossusSmash).Remains() > 4.5 and not Azerite.TestOfMight.Enabled()) or Player.Buff(Aura.TestOfMight).Up());
		end);
		-- actions.five_target+=/deadly_calm
		action.EvaluateAction(Talent.DeadlyCalm, true);
		-- actions.five_target+=/cleave
		action.EvaluateAction(Talent.Cleave, true);
		-- We are not doing legion legendaries as they stop working at 115, so just skip those checks.
		-- actions.five_target+=/execute,if=(!talent.cleave.enabled&dot.deep_wounds.remains<2)|(buff.sudden_death.react|buff.stone_heart.react)&(buff.sweeping_strikes.up|cooldown.sweeping_strikes.remains>8)
		action.EvaluateAction(Spell.Execute, function()
			return (not Talent.Cleave.Enabled() and Target.Debuff(Aura.DeepWounds).Remains() < 2)
			    or Player.Buff(Aura.SuddenDeath).React()
			   and (Player.Buff(Aura.SweepingStrikes).Up() or Spell.SweepingStrikes.Cooldown.Remains() > 8);
		end);
		-- actions.five_target+=/mortal_strike,if=(!talent.cleave.enabled&dot.deep_wounds.remains<2)|buff.sweeping_strikes.up&buff.overpower.stack=2&(talent.dreadnaught.enabled|buff.executioners_precision.stack=2)
		action.EvaluateAction(Spell.MortalStrike, function()
			return (not Talent.Cleave.Enabled() and Target.Debuff(Aura.DeepWounds).Remains() < 2)
			    or Player.Buff(Aura.SweepingStrikes).Up()
			   and Player.Buff(Aura.Overpower).Stack() == 2
			   and (Talent.Dreadnaught.Enabled() or Player.Buff(Aura.ExecutionersPrecision).Stack() == 2);
		end);
		-- actions.five_target+=/whirlwind,if=debuff.colossus_smash.up|(buff.crushing_assault.up&talent.fervor_of_battle.enabled)
		action.EvaluateAction(Spell.Whirlwind, function()
			return Target.Debuff(Aura.ColossusSmash).Up()
			    or (Player.Buff(Aura.CrushingAssault).Up() and Talent.FervorOfBattle.Enabled());
		end);
		--actions.five_target+=/whirlwind,if=buff.deadly_calm.up|rage>60
		action.EvaluateAction(Spell.Whirlwind, function()
			return Target.Debuff(Aura.DeadlyCalm).Up()
			    or Player.Rage() > 60;
		end);
		-- actions.five_target+=/overpower
		action.EvaluateAction(Spell.Overpower, true);
		-- actions.five_target+=/whirlwind
		action.EvaluateAction(Spell.Whirlwind, true);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationAOE = AOE("AOE");

-- Execute Rotation
local function Execute(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.execute=skullsplitter,if=rage<60&(!talent.deadly_calm.enabled|buff.deadly_calm.down)
		action.EvaluateAction(Talent.Skullsplitter, function()
			return Player.Rage() < 60
					and (Talent.DeadlyCalm.Enabled() or Player.Buff(Aura.DeadlyCalm).Down());
		end);
		-- actions.execute+=/ravager,if=!buff.deadly_calm.up&(cooldown.colossus_smash.remains<2|(talent.warbreaker.enabled&cooldown.warbreaker.remains<2))
		action.EvaluateAction(Talent.Ravager, function()
			return not Player.Buff(Aura.DeadlyCalm).Up()
			   and (Spell.ColossusSmash.Cooldown.Remains() < 2 or (Talent.Warbreaker.Enabled() and Talent.Warbreaker.Cooldown.Remains() < 2));
		end);
		-- actions.execute+=/colossus_smash,if=debuff.colossus_smash.down
		action.EvaluateAction(Spell.ColossusSmash, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.execute+=/warbreaker,if=debuff.colossus_smash.down
		action.EvaluateAction(Talent.Warbreaker, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.execute+=/deadly_calm
		action.EvaluateAction(Talent.DeadlyCalm, true);
		-- actions.execute+=/bladestorm,if=rage<30&!buff.deadly_calm.up
		action.EvaluateAction(Spell.Bladestorm, function()
			return Player.Rage() < 30
			   and not Player.Buff(Aura.DeadlyCalm).Up();
		end);
		-- actions.execute+=/cleave,if=spell_targets.whirlwind>2
		action.EvaluateAction(Talent.Cleave, function(numEnemies)
			return numEnemies > 2;
		end, 8);
		-- actions.execute+=/slam,if=buff.crushing_assault.up
		action.EvaluateAction(Spell.Slam, function()
			return Player.Buff(Aura.CrushingAssault).Up()
		end);
		-- actions.execute+=/mortal_strike,if=buff.overpower.stack=2&talent.dreadnaught.enabled|buff.executioners_precision.stack=2
		action.EvaluateAction(Spell.MortalStrike, function()
			return Player.Buff(Aura.Overpower).Stack() == 2
			   and Talent.Dreadnaught.Enabled()
			    or Player.Buff(Aura.ExecutionersPrecision).Stack() == 2;
		end);
		-- actions.execute+=/execute,if=buff.deadly_calm.up
		action.EvaluateAction(Spell.Execute, function()
			return Player.Buff(Aura.DeadlyCalm).Up();
		end);
		-- actions.execute+=/overpower
		action.EvaluateAction(Spell.Overpower, true);
		-- actions.execute+=/execute
		action.EvaluateAction(Spell.Execute, true);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationExecute = Execute("Execute");

-- HAC Rotation
local function HAC(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	-- Table to hold requirements to use spells and items listed in the rotation.
	self.Requirements = {};

	Objects.FinalizeRequirements(self.Requirements);

	-- This subrotation only runs if raid adds are up, that is number of targets is higher then desired targets. (boss frames)
	-- So skip any condition checks regarding raid.adds.up.
	function self.Rotation(action)
		-- remains<=duration*0.3 is the same as Refreshable().
		-- actions.hac=rend,if=remains<=duration*0.3&(!raid_event.adds.up|buff.sweeping_strikes.up)
		action.EvaluateAction(Talent.Rend, function()
			return Target.Debuff(Aura.Rend).Refreshable()
			   and Player.Buff(Aura.SweepingStrikes).Up();
		end);
		-- actions.hac+=/skullsplitter,if=rage<60&(cooldown.deadly_calm.remains>3|!talent.deadly_calm.enabled)
		action.EvaluateAction(Talent.Skullsplitter, function()
			return Player.Rage() < 60
			   and (Talent.DeadlyCalm.Cooldown.Remains() > 3 or not Talent.DeadlyCalm.Enabled());
		end);
		-- actions.hac+=/deadly_calm,if=(cooldown.bladestorm.remains>6|talent.ravager.enabled&cooldown.ravager.remains>6)&(cooldown.colossus_smash.remains<2|(talent.warbreaker.enabled&cooldown.warbreaker.remains<2))
		action.EvaluateAction(Talent.DeadlyCalm, function()
			return (Spell.Bladestorm.Cooldown.Remains() > 6 or Talent.Ravager.Enabled() and Talent.Ravager.Cooldown.Remains() > 6)
			   and (Spell.ColossusSmash.Cooldown.Remains() < 2 or (Talent.Warbreaker.Enabled() and Talent.Warbreaker.Cooldown.Remains() < 2));
		end);
		-- actions.hac+=/ravager,if=(raid_event.adds.up|raid_event.adds.in>target.time_to_die)&(cooldown.colossus_smash.remains<2|(talent.warbreaker.enabled&cooldown.warbreaker.remains<2))
		action.EvaluateAction(Talent.Ravager, function()
			return Spell.ColossusSmash.Cooldown.Remains() < 2
			    or (Talent.Warbreaker.Enabled() and Talent.Warbreaker.Cooldown.Remains() < 2);
		end);
		-- The first condition must be true, so there is no need putting in a check for the others as it will be suggested anyways
		-- actions.hac+=/colossus_smash,if=raid_event.adds.up|raid_event.adds.in>40|(raid_event.adds.in>20&talent.anger_management.enabled)
		action.EvaluateAction(Spell.ColossusSmash, true);
		-- The first condition must be true, so there is no need putting in a check for the others as it will be suggested anyways
		-- actions.hac+=/warbreaker,if=raid_event.adds.up|raid_event.adds.in>40|(raid_event.adds.in>20&talent.anger_management.enabled)
		action.EvaluateAction(Talent.Warbreaker, true);
		-- actions.hac+=/bladestorm,if=(debuff.colossus_smash.up&raid_event.adds.in>target.time_to_die)|raid_event.adds.up&((debuff.colossus_smash.remains>4.5&!azerite.test_of_might.enabled)|buff.test_of_might.up)
		action.EvaluateAction(Spell.Bladestorm, function()
			return (Target.Debuff(Aura.ColossusSmash).Remains() > 4.5 and not Azerite.TestOfMight.Enabled())
			    or Player.Buff(Aura.TestOfMight).Up();
		end);
		-- actions.hac+=/overpower,if=!raid_event.adds.up|(raid_event.adds.up&azerite.seismic_wave.enabled)
		action.EvaluateAction(Spell.Overpower, function()
			return Azerite.SeismicWave.Enabled();
		end);
		-- actions.hac+=/cleave,if=spell_targets.whirlwind>2
		action.EvaluateAction(Talent.Cleave, function(numEnemies)
			return numEnemies > 2;
		end, 8);
		-- actions.hac+=/execute,if=!raid_event.adds.up|(!talent.cleave.enabled&dot.deep_wounds.remains<2)|buff.sudden_death.react
		action.EvaluateAction(Spell.Execute, function()
			return (not Talent.Cleave.Enabled() and Target.Debuff(Aura.DeepWounds).Remains() <2)
			    or Player.Buff(Aura.SuddenDeath).React();
		end);
		-- actions.hac+=/mortal_strike,if=!raid_event.adds.up|(!talent.cleave.enabled&dot.deep_wounds.remains<2)
		action.EvaluateAction(Spell.MortalStrike, function()
			return not Talent.Cleave.Enabled()
			   and Target.Debuff(Aura.DeepWounds).Remains() < 2;
		end);
		-- actions.hac+=/whirlwind,if=raid_event.adds.up
		action.EvaluateAction(Spell.Whirlwind, true);
		-- actions.hac+=/overpower
		action.EvaluateAction(Spell.Overpower, true);
		-- actions.hac+=/whirlwind,if=talent.fervor_of_battle.enabled
		action.EvaluateAction(Spell.Whirlwind, function()
			return Talent.FervorOfBattle.Enabled();
		end);
		-- actions.hac+=/slam,if=!talent.fervor_of_battle.enabled&!raid_event.adds.up
		action.EvaluateAction(Spell.Slam, function(numEnemies)
			return not Talent.FervorOfBattle.Enabled();
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationHAC = HAC("HAC");

-- SingleTarget Rotation
local function SingleTarget(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- remains<=duration*0.3 is the same as Refreshable().
		-- actions.single_target=rend,if=remains<=duration*0.3&debuff.colossus_smash.down
		action.EvaluateAction(Talent.Rend, function()
			return Target.Debuff(Aura.Rend).Refreshable()
			   and Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.single_target+=/skullsplitter,if=rage<60&(!talent.deadly_calm.enabled|buff.deadly_calm.down)
		action.EvaluateAction(Talent.Skullsplitter, function()
			return Player.Rage() < 60
			   and (not Talent.DeadlyCalm.Enabled() or Player.Buff(Aura.DeadlyCalm).Down());
		end);
		--actions.single_target+=/ravager,if=!buff.deadly_calm.up&(cooldown.colossus_smash.remains<2|(talent.warbreaker.enabled&cooldown.warbreaker.remains<2))
		action.EvaluateAction(Talent.Ravager, function()
			return not Player.Buff(Aura.DeadlyCalm).Up()
			   and (Spell.ColossusSmash.Cooldown.Remains() <2 or (Talent.Warbreaker.Enabled() and Talent.Warbreaker.Cooldown.Remains() < 2));
		end);
		-- actions.single_target+=/colossus_smash,if=debuff.colossus_smash.down
		action.EvaluateAction(Spell.ColossusSmash, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.single_target+=/warbreaker,if=debuff.colossus_smash.down
		action.EvaluateAction(Talent.Warbreaker, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions.single_target+=/deadly_calm
		action.EvaluateAction(Talent.DeadlyCalm, true);
		-- actions.single_target+=/execute,if=buff.sudden_death.react
		action.EvaluateAction(Spell.Execute, function()
			return Player.Buff(Aura.SuddenDeath).React();
		end);
		-- Lua can't evaluate a number as true and 0 as false, so we need to use a boolean return, so use Down()
		-- actions.single_target+=/bladestorm,if=cooldown.mortal_strike.remains&(!talent.deadly_calm.enabled|buff.deadly_calm.down)&((debuff.colossus_smash.up&!azerite.test_of_might.enabled)|buff.test_of_might.up)
		action.EvaluateAction(Spell.Bladestorm, function()
			return Spell.MortalStrike.Cooldown.Down()
			   and (not Talent.DeadlyCalm.Enabled() or Player.Buff(Aura.DeadlyCalm).Down())
			   and ((Target.Debuff(Aura.ColossusSmash).Up() and not Azerite.TestOfMight.Enabled()) or Player.Buff(Aura.TestOfMight).Up());
		end);
		-- actions.single_target+=/cleave,if=spell_targets.whirlwind>2
		action.EvaluateAction(Talent.Cleave, function(numEnemies)
			return numEnemies > 2;
		end, 8);
		-- actions.single_target+=/overpower,if=azerite.seismic_wave.rank=3
		action.EvaluateAction(Spell.Overpower, function()
			return Azerite.SeismicWave.Rank() == 3;
		end);
		-- actions.single_target+=/mortal_strike
		action.EvaluateAction(Spell.MortalStrike, true);
		-- actions.single_target+=/whirlwind,if=talent.fervor_of_battle.enabled&(buff.deadly_calm.up|rage>=60)
		action.EvaluateAction(Spell.Whirlwind, function()
			return Talent.FervorOfBattle.Enabled()
			   and (Player.Buff(Aura.DeadlyCalm).Up() or Player.Rage() >= 60);
		end);
		-- actions.single_target+=/overpower
		action.EvaluateAction(Spell.Overpower, true);
		-- actions.single_target+=/whirlwind,if=talent.fervor_of_battle.enabled&(!azerite.test_of_might.enabled|debuff.colossus_smash.up)
		action.EvaluateAction(Spell.Whirlwind, function()
			return Talent.FervorOfBattle.Enabled()
			   and (not Azerite.TestOfMight.Enabled() or Target.Debuff(Aura.ColossusSmash).Up());
		end);
		-- actions.single_target+=/slam,if=!talent.fervor_of_battle.enabled&(!azerite.test_of_might.enabled|debuff.colossus_smash.up|buff.deadly_calm.up|rage>=60)
		action.EvaluateAction(Spell.Slam, function()
			return not Talent.FervorOfBattle.Enabled()
			   and (not Azerite.TestOfMight.Enabled() or Target.Debuff(Aura.ColossusSmash).Up() or Player.Buff(Aura.DeadlyCalm).Up() or Player.Rage() >= 60);
		end);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationSingleTarget = SingleTarget("SingleTarget");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
	-- Inherits APL Class so get the base class.
	local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

	-- Store the information for the script.
	self.scriptInfo = {
		SpecializationID = self.SpecID,
		ScriptAuthor = "LunaEclipse",
		GuideAuthor = "Archimtiros and SimCraft",
		GuideLink = "https://www.wowhead.com/arms-warrior-guide",
		WoWVersion = 80001,
	};

	-- Table to hold requirements to use spells for defensive reasons.
	self.Defensives = {
		DefensiveStance = function()
			return Player.DamagePredicted(4) >= 80;
		end,

		DieByTheSword = function()
			return Player.PhysicalDamagePredicted(4) >= 60
			    or Player.MagicDamagePredicted(4) >= 20;
		end,
	};

	Objects.FinalizeRequirements(self.Defensives);

	-- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
	function self.Enable(...)
		Racial, Spell, Talent, Aura, Azerite = ...;

		Aura.CrushingAssault = 278826;
		Aura.ExecutionersPrecision = 242188;
		Aura.TestOfMight = 275540;

		Item = {};

		Consumable = {
			-- Potions
			BattlePotionOfStrength = Objects.newItem(163224),
		};

		Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for setting up the configuration screen, called when rotation becomes the active rotation.
	function self.SetupConfiguration(config, options)
		config.RacialOptions(options, Racial.AncestralCall, Racial.ArcaneTorrent, Racial.Berserking, Racial.BloodFury, Racial.Fireblood, Racial.LightsJudgment);
		config.AOEOptions(options, Spell.Bladestorm, Talent.Cleave, Talent.Ravager, Spell.SweepingStrikes, Spell.Whirlwind);
		config.CooldownOptions(options, Talent.Avatar, Talent.DeadlyCalm, Spell.Execute, Talent.Rend, Talent.Skullsplitter, Talent.Warbreaker);
		config.DefensiveOptions(options, Talent.DefensiveStance, Spell.DieByTheSword);
		config.UtilityOptions(options, Spell.Charge, Spell.HeroicLeap);
	end

	-- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
	function self.Disable()
		local coreTables = addon.Core.Tables;

		coreTables.Wipe(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for checking the rotation that displays on the Defensives icon.
	function self.Defensive(action)
		-- The abilities here should be listed from highest damage required to suggest to lowest,
		-- Specific damage types before all damage types.

		-- Protects against all types of damage
		action.EvaluateDefensiveAction(Spell.DieByTheSword, self.Defensives.DieByTheSword);
		action.EvaluateDefensiveAction(Talent.DefensiveStance, self.Defensives.DefensiveStance);
	end

	-- Function for displaying opening rotation.
	function self.Opener(action)
	end

	-- Function for displaying any actions before combat starts.
	function self.Precombat(action)
		-- actions.precombat+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfStrength, true);
	end

	-- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
	function self.Combat(action)
		-- actions=charge
		action.EvaluateAction(Spell.Charge, true);
		-- Suggest this when the target is out of melee range but within heroic leap range.
		action.EvaluateAction(Spell.HeroicLeap, function()
			return Target.InRange.Max() >= 8
			   and Target.InRange.Max() <= 40;
		end);
		-- actions+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfStrength, true);
		-- actions+=/blood_fury,if=debuff.colossus_smash.up
		action.EvaluateAction(Racial.BloodFury, function()
			return Target.Debuff(Aura.ColossusSmash).Up();
		end);
		-- actions+=/berserking,if=debuff.colossus_smash.up
		action.EvaluateAction(Racial.Berserking, function()
			return Target.Debuff(Aura.ColossusSmash).Up();
		end);
		-- actions+=/arcane_torrent,if=debuff.colossus_smash.down&cooldown.mortal_strike.remains>1.5&rage<50
		action.EvaluateAction(Racial.ArcaneTorrent, function()
			return Target.Debuff(Aura.ColossusSmash).Down()
			   and Spell.MortalStrike.Cooldown.Remains() > 1.5
			   and Player.Rage() < 50;
		end);
		-- actions+=/lights_judgment,if=debuff.colossus_smash.down
		action.EvaluateAction(Racial.LightsJudgment, function()
			return Target.Debuff(Aura.ColossusSmash).Down();
		end);
		-- actions+=/fireblood,if=debuff.colossus_smash.up
		action.EvaluateAction(Racial.Fireblood, function()
			return Target.Debuff(Aura.ColossusSmash).Up();
		end);
		-- actions+=/ancestral_call,if=debuff.colossus_smash.up
		action.EvaluateAction(Racial.AncestralCall, function()
			return Target.Debuff(Aura.ColossusSmash).Up();
		end);
		-- actions+=/avatar,if=cooldown.colossus_smash.remains<8|(talent.warbreaker.enabled&cooldown.warbreaker.remains<8)
		action.EvaluateAction(Talent.Avatar, function()
			return Spell.ColossusSmash.Cooldown.Remains() < 8
			    or (Talent.Warbreaker.Enabled() and Talent.Warbreaker.Cooldown.Remains() < 8);
		end);
		-- actions+=/sweeping_strikes,if=spell_targets.whirlwind>1&(cooldown.bladestorm.remains>10|cooldown.colossus_smash.remains>8|azerite.test_of_might.enabled)
		action.EvaluateAction(Spell.SweepingStrikes, function(numEnemies)
			return numEnemies > 1
			   and (Spell.Bladestorm.Cooldown.Remains() > 10 or Spell.ColossusSmash.Cooldown.Remains() > 8 or Azerite.TestOfMight.Enabled());
		end, 8);

		-- actions+=/run_action_list,name=hac,if=raid_event.adds.exists
		action.RunActionList(rotationHAC, function(numEnemies)
			return coreGeneral.AddsUp();
		end);
		-- actions+=/run_action_list,name=five_target,if=spell_targets.whirlwind>4
		action.RunActionList(rotationAOE, function()
			return Enemies.GetEnemies(8) > 4;
		end);
		-- actions+=/run_action_list,name=execute,if=(talent.massacre.enabled&target.health.pct<35)|target.health.pct<20
		action.RunActionList(rotationExecute, function()
			return (Talent.Massacre.Enabled() and Target.Health.Percent() < 35)
				or Target.Health.Percent() < 20;
		end);
		-- actions+=/run_action_list,name=single_target
		action.RunActionList(rotationSingleTarget);
	end

	return self;
end

local rotationAPL = APL(nameAPL, "LunaEclipse: Arms Warrior", addonTable.Enum.SpecID.WARRIOR_ARMS);
