local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

--- Localize Vars
local coreSettings = addon.Core.Settings;
local Objects = addon.Core.Objects;

-- Objects
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "lunaeclipse_warrior_fury";

-- SingleTarget Rotation
local function SingleTarget(rotationName)
	-- Inherits Rotation Class so get the base class.
	local self = addonTable.rotationsClass(nameAPL, rotationName);

	function self.Rotation(action)
		-- actions.single_target=siegebreaker
		action.EvaluateAction(Talent.Siegebreaker, true);
		-- actions.single_target+=/rampage,if=buff.recklessness.up|(talent.frothing_berserker.enabled|talent.carnage.enabled&(buff.enrage.remains<gcd|rage>90)|talent.massacre.enabled&(buff.enrage.remains<gcd|rage>90))
		action.EvaluateAction(Spell.Rampage, function()
			return Player.Buff(Aura.Recklessness).Up()
			    or (Talent.FrothingBerserker.Enabled() or Talent.Carnage.Enabled() and (Player.Buff(Aura.Enrage).Remains() < Player.GCD() or Player.Rage() > 90) or Talent.Massacre.Enabled() and (Player.Buff(Aura.Enrage).Remains() < Player.GCD() or Player.Rage() > 90));
		end);
		-- actions.single_target+=/execute,if=buff.enrage.up
		action.EvaluateAction(Spell.Execute, function()
			return Player.Buff(Aura.Enrage).Up();
		end);
		-- actions.single_target+=/bloodthirst,if=buff.enrage.down
		action.EvaluateAction(Spell.Bloodthirst, function()
			return Player.Buff(Aura.Enrage).Down();
		end);
		-- actions.single_target+=/raging_blow,if=charges=2
		action.EvaluateAction(Spell.RagingBlow, function()
			return Spell.RagingBlow.Charges() == 2;
		end);
		-- actions.single_target+=/bloodthirst
		action.EvaluateAction(Spell.Bloodthirst, true);
		-- The PrevGCD Ramage and debuff up is not practical for a human being, its almost never suggested and when it is it flashes for one refresh cycle only.
		-- Change this to buff.enrage.up instead to allow more time for a human being to use it.
		-- actions.single_target+=/bladestorm,if=prev_gcd.1.rampage&(debuff.siegebreaker.up|!talent.siegebreaker.enabled)
		action.EvaluateAction(Talent.Bladestorm, function()
			return Player.Buff(Aura.Enrage).Up()
			   and (Target.Debuff(Aura.Siegebreaker).Up() or not Talent.Siegebreaker.Enabled());
		end);
		-- actions.single_target+=/dragon_roar,if=buff.enrage.up
		action.EvaluateAction(Talent.DragonRoar, function()
			return Player.Buff(Aura.Enrage).Up();
		end);
		-- actions.single_target+=/raging_blow,if=talent.carnage.enabled|(talent.massacre.enabled&rage<80)|(talent.frothing_berserker.enabled&rage<90)
		action.EvaluateAction(Spell.RagingBlow, function()
			return Talent.Carnage.Enabled()
			    or (Talent.Massacre.Enabled() and Player.Rage() < 80)
			    or (Talent.FrothingBerserker.Enabled() and Player.Rage() < 90);
		end);
		-- actions.single_target+=/furious_slash,if=talent.furious_slash.enabled
		action.EvaluateAction(Talent.FuriousSlash, function()
			return Talent.FuriousSlash.Enabled();
		end);
		-- actions.single_target+=/whirlwind
		action.EvaluateAction(Spell.Whirlwind, true);
	end

	return self;
end

-- Create a variable so we can call the rotations functions.
local rotationSingleTarget = SingleTarget("SingleTarget");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
	-- Inherits APL Class so get the base class.
	local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

	-- Store the information for the script.
	self.scriptInfo = {
		SpecializationID = self.SpecID,
		ScriptAuthor = "LunaEclipse",
		GuideAuthor = "Archimtiros and SimCraft",
		GuideLink = "https://www.wowhead.com/fury-warrior-guide",
		WoWVersion = 80001,
	};

	-- Table to hold requirements to use spells for defensive reasons.
	self.Defensives = {
		Bloodthirst = function()
			return Player.Health.Percent() < 90
			   and Player.Buff(Aura.EnragedRegeneration).Up();
		end,

		EnragedRegeneration = function()
			return Player.DamagePredicted(4) >= 30;
		end,
	};

	Objects.FinalizeRequirements(self.Defensives);

	-- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
	function self.Enable(...)
		Racial, Spell, Talent, Aura, Azerite = ...;

		Item = {};

		Consumable = {
			-- Potions
			BattlePotionOfStrength = Objects.newItem(163224),
		};

		Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for setting up the configuration screen, called when rotation becomes the active rotation.
	function self.SetupConfiguration(config, options)
		config.RacialOptions(options, Racial.AncestralCall, Racial.ArcaneTorrent, Racial.Berserking, Racial.BloodFury, Racial.Fireblood, Racial.LightsJudgment);
		config.AOEOptions(options, Talent.Bladestorm, Spell.Whirlwind);
		config.CooldownOptions(options, Talent.DragonRoar, Talent.FuriousSlash, Spell.Rampage, Spell.RagingBlow, Spell.Recklessness, Talent.Siegebreaker);
		config.DefensiveOptions(options, "Bloodthirst|Use Bloodthirst when you have the Enraged Regeneration buff for self healing.", Spell.EnragedRegeneration);
		config.UtilityOptions(options, Spell.Charge, Spell.HeroicLeap);
	end

	-- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
	function self.Disable()
		local coreTables = addon.Core.Tables;

		coreTables.Wipe(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
	end

	-- Function for checking the rotation that displays on the Defensives icon.
	function self.Defensive(action)
		-- The abilities here should be listed from highest damage required to suggest to lowest,
		-- Specific damage types before all damage types.

		-- Protects against all types of damage
		action.EvaluateDefensiveAction(Spell.EnragedRegeneration, self.Defensives.EnragedRegeneration);

		-- Self Healing goes at the end and is only suggested if a major cooldown is not needed.
		action.EvaluateDefensiveAction(Spell.Bloodthirst, self.Defensives.Bloodthirst);
	end

	-- Function for displaying opening rotation.
	function self.Opener(action)
	end

	-- Function for displaying any actions before combat starts.
	function self.Precombat(action)
		-- actions.precombat+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfStrength, true);
	end

	-- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
	function self.Combat(action)
		-- actions+=/charge
		action.EvaluateAction(Spell.Charge, true);
		-- We can't do raid raid movement events, suggest this when the target is out of melee range but within heroic leap range.
		-- actions+=/heroic_leap,if=(raid_event.movement.distance>25&raid_event.movement.in>45)|!raid_event.movement.exists
		action.EvaluateAction(Spell.HeroicLeap, function()
			return Target.InRange.Max() >= 8
			   and Target.InRange.Max() <= 40;
		end);
		-- actions+=/potion
		action.EvaluateAction(Consumable.BattlePotionOfStrength, true);
		-- actions+=/furious_slash,if=talent.furious_slash.enabled&(buff.furious_slash.stack<3|buff.furious_slash.remains<3|(cooldown.recklessness.remains<3&buff.furious_slash.remains<9))
		action.EvaluateAction(Talent.FuriousSlash, function()
			return Talent.FuriousSlash.Enabled()
			   and (Player.Buff(Aura.FuriousSlash).Stack() < 3 or Player.Buff(Aura.FuriousSlash).Remains() < 3 or (Spell.Recklessness.Cooldown.Remains() < 3 and Player.Buff(Aura.FuriousSlash).Remains() < 9));
		end);
		-- actions+=/rampage,if=cooldown.recklessness.remains<3
		action.EvaluateAction(Spell.Rampage, function()
			return Spell.Recklessness.Cooldown.Remains() < 3;
		end);
		-- actions+=/recklessness
		action.EvaluateAction(Spell.Recklessness, true);
		-- SimCraft doesn't specify it, but only check Meat Cleaver buff if you chose Meat Cleaver talent, otherwise only suggest if you don't already have the buff.
		-- actions+=/whirlwind,if=spell_targets.whirlwind>1&!buff.meat_cleaver.up
		action.EvaluateAction(Spell.Whirlwind, function(numEnemies)
			return numEnemies > 1
			   and ((Talent.MeatCleaver.Enabled() and not Player.Buff(Aura.MeatCleaver).Up()) or Player.Buff(Aura.Whirlwind).Down());
		end, 8);
		-- actions+=/blood_fury,if=buff.recklessness.up
		action.EvaluateAction(Racial.BloodFury, function()
			return Player.Buff(Aura.Recklessness).Up();
		end);
		-- actions+=/berserking,if=buff.recklessness.up
		action.EvaluateAction(Racial.Berserking, function()
			return Player.Buff(Aura.Recklessness).Up();
		end);
		-- actions+=/lights_judgment,if=buff.recklessness.down
		action.EvaluateAction(Racial.LightsJudgment, function()
			return Player.Buff(Aura.Recklessness).Down();
		end);
		-- actions+=/fireblood,if=buff.recklessness.up
		action.EvaluateAction(Racial.Fireblood, function()
			return Player.Buff(Aura.Recklessness).Up();
		end);
		-- actions+=/ancestral_call,if=buff.recklessness.up
		action.EvaluateAction(Racial.AncestralCall, function()
			return Player.Buff(Aura.Recklessness).Up();
		end);

		-- actions+=/run_action_list,name=single_target
		action.RunActionList(rotationSingleTarget);
	end

	return self;
end

local rotationAPL = APL(nameAPL, "LunaEclipse: Fury Warrior", addonTable.Enum.SpecID.WARRIOR_FURY);
