-- Pulls back the Addon-Local Variables and store them locally.
local _, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];

-- Plugin information variables.
local pluginName = "Game Tweaks";
local pluginDescription = "This plugin allows the user to tweak their World of Warcraft from cleaning addon memory when out of combat to improve game performance or improved quality of life features.\n\nAt present only the addon memory cleaner is included, more quality of life improvements will be added later!";

-- Check for LunaEclipse Rotation Helper being loaded first so it doesn't register the plugin if its not needed.
if addon then
    local optionsTable = {
        type = "group",
        name = pluginName,
        get = function(info)
            local returnValue = 0;

            if info then
                local Tables = addon.Core.Tables;
                local profile = addonTable.gameTweaks;
                local keyName = info[#info];

                returnValue = Tables.GetValue(profile, keyName);

                if returnValue == nil then
                    returnValue = 0;
                    profile[keyName] = returnValue;

                    addonTable.gameTweaks = profile;
                end

                if info.option.type == "toggle" then
                    returnValue = addon.Options.Core.ConvertToUI(returnValue, info.option.tristate);
                end
            end

            return returnValue;
        end,
        set = function(info, value)
            if info then
                local profile = addonTable.gameTweaks;
                local keyName = info[#info];

                if info.option.type == "toggle" then
                    value = addon.Options.Core.ConvertToValue(value, info.option.tristate);
                end

                if keyName == "useMemoryCleaner" then
                    if value == 1 then
                        addon.Features.MemoryCleanup.Start(true);
                    else
                        addon.Features.MemoryCleanup.Stop(true);
                    end
                end

                profile[keyName] = value;

                -- Put the settings into the database
                addonTable.gameTweaks = profile;
            end
        end,
        childGroups = "tree",
        args = {
            useMemoryCleaner = {
                type = "toggle",
                name = "Automatically clean addon memory when not in combat.",
                desc = "This will automatically clean addon memory when not in combat to remove any objects no longer needed from memory.\n\nMemory cleaning can improve FPS on any computer though it will have more effect on older computer systems with less memory or slower memory.\n\nRecommended: Checked!",
                width = "full",
                order = 10,
            },
            cleanupRate = {
                type = "range",
                name = "Memory Cleanup Time",
                desc = "Select the amount of time between each time the addon memory cleanup is executed.\n\nA lower value will help older computers or computers with less memory, for newer computers this can be much higher. This is measured in seconds.\n\nMemory cleanup can cause your computer to not respond for a brief period, so it is suggested to use as large a value as possible while still maintaining playability!",
                min = 5,
                max = 60,
                step = 5,
                width = "full",
                order = 20,
            },
        },
    };

    -- The plugin still has to be registered even if its not loaded into memory, otherwise there is no checkbox to re-enable it.
    addon.System.Plugins.RegisterPlugin(pluginName, pluginDescription, optionsTable);
end
