-- Pulls back the Addon-Local Variables and store them locally.
local _, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];
local pluginName = "SpellFlash";

-- Check for LunaEclipse Rotation Helper being loaded first so it doesn't register the plugin if its not needed.
if addon and addon.System.Plugins.IsActive(pluginName) then
	-- Store local copies of Blizzard API and other addon global functions and variables
	local ATTACK_BUTTON_FLASH_TIME = ATTACK_BUTTON_FLASH_TIME;
	local format, strlower, type = format, strlower, type;

	local colorTable = {
		aqua = { r = 0, g = 1, b = 1 },
		blue = { r = 0, g = 0, b = 1 },
		gray = { r = 0.5, g = 0.5, b = 0.5 },
		green = { r = 0.1, g = 1, b = 0.1 },
		orange = { r = 1, g = 0.5, b = 0.25 },
		pink = { r = 0.9, g = 0.4, b = 0.4 },
		purple = { r = 1, g = 0, b = 1 },
		red = { r = 1, g = 0.1, b = 0.1 },
		white = { r = 1, g = 1, b = 1 },
		yellow = { r = 1, g = 1, b = 0 },
	};

	local flashInfo = {};
	local flashFrames = {};

	local eventFrame = addon.API.Blizzard.CreateFrame("Frame");
	local flashFrame = "LERH_ActionFlash";

	local Overlay = {};

	Overlay = {
		Create = function(actionButton)
			local buttonName = actionButton ~= nil and actionButton:GetName();

			if buttonName then
				local overlayName = format("%s_%s", flashFrame, buttonName);

				if not flashFrames[buttonName] then
					flashFrames[buttonName] = addon.API.Blizzard.CreateFrame("Frame", overlayName, actionButton);
					flashFrames[buttonName]:SetAllPoints(actionButton);
					flashFrames[buttonName]:SetFrameStrata("DIALOG");
					flashFrames[buttonName]:EnableMouse(false);
					flashFrames[buttonName]:Hide();

					flashFrames[buttonName].FlashTexture = flashFrames[buttonName]:CreateTexture(nil, "OVERLAY");
					flashFrames[buttonName].FlashTexture:SetTexture("Interface\\Cooldown\\star4");
					flashFrames[buttonName].FlashTexture:SetPoint("CENTER", flashFrames[buttonName], "CENTER");
					flashFrames[buttonName].FlashTexture:SetBlendMode("ADD");
					flashFrames[buttonName].FlashTexture:SetVertexColor(colorTable.white.r, colorTable.white.g, colorTable.white.b);
					flashFrames[buttonName].FlashTexture:SetAlpha(1);
				end

				return flashFrames[buttonName];
			end
		end,

		Get = function(buttonID)
			local actionButton = addon.Modules.ActionBar.Button(buttonID);
			local buttonName = actionButton ~= nil and actionButton:GetName();

			if buttonName then
				return flashFrames[buttonName] or Overlay.Create(actionButton);
			end
		end,

		IsFlashing = function(buttonID)
			local buttonOverlay = Overlay.Get(buttonID);

			if buttonOverlay then
				return buttonOverlay.flashing == 1;
			end

			return false;
		end,

		IsVisible = function(buttonID)
			local buttonOverlay = Overlay.Get(buttonID);

			if buttonOverlay then
				return buttonOverlay:IsShown();
			end

			return false;
		end,

		Start = function(buttonID, color)
			local coreTables = addon.Core.Tables;

			if buttonID and addon.Modules.ActionBar.Active(buttonID) and not Overlay.IsFlashing(buttonID) then
				local buttonOverlay = Overlay.Get(buttonID);

				if buttonOverlay then
					flashInfo[buttonID] = flashInfo[buttonID]
							           or (type(color) == "table" and color)
							           or (type(color) == "string" and colorTable[strlower(color)])
						               or colorTable.white;
					local flashColor = coreTables.GetValue(flashInfo, buttonID, colorTable.white);

					buttonOverlay.FlashTexture:SetVertexColor(flashColor.r, flashColor.g, flashColor.b);
					buttonOverlay.flashing = 1;

					return true;
				end
			end

			return false;
		end,

		Stop = function(buttonID)
			if buttonID and addon.Modules.ActionBar.Active(buttonID) and Overlay.IsFlashing(buttonID) then
				local buttonOverlay = Overlay.Get(buttonID);

				if buttonOverlay then
					flashInfo[buttonID] = nil;

					buttonOverlay.flashing = 0;

					return true;
				end
			end

			return false;
		end,
	};

	addon.Features.SpellFlash = {
		Flash = function(actionType, actionID, color)
			local buttonID = addon.Modules.ActionBar.ButtonID(actionType, actionID);

			if buttonID then
				return Overlay.Start(buttonID, color);
			end

			return false;
		end,

		Stop = function(actionType, actionID)
			local buttonID = addon.Modules.ActionBar.ButtonID(actionType, actionID);

			if buttonID then
				return Overlay.Stop(buttonID);
			end

			return false;
		end,
	};

	local flashTime = 0;

	local function onUpdate(_, elapsed)
		flashTime = flashTime - elapsed;

		-- Run the loop only when there is something to update
		if flashTime <= 0 then
			for buttonID = 1, 120 do
				local buttonOverlay = Overlay.Get(buttonID);

				if buttonOverlay then
					if addon.Modules.ActionBar.Active(buttonID) and Overlay.IsFlashing(buttonID) then
						if Overlay.IsVisible(buttonID) then
							buttonOverlay:Hide();
						else
							buttonOverlay:Show();
						end
					elseif not addon.Modules.ActionBar.Active(buttonID) then
						Overlay.Stop(buttonID);
					elseif not Overlay.IsFlashing(buttonID) then
						buttonOverlay:Hide();
						buttonOverlay.FlashTexture:SetVertexColor(colorTable.white.r, colorTable.white.g, colorTable.white.b);
					end
				end
			end

			-- Update values
			flashTime = flashTime + ATTACK_BUTTON_FLASH_TIME;
		end
	end

	local moduleEvents = {
		playerLogin = function()
			for buttonID = 1, 120 do
				Overlay.Get(buttonID);
			end
		end,

		Disabled = function()
			eventFrame:SetScript("OnUpdate", nil);

			for buttonID = 1, 120 do
				Overlay.Stop(buttonID);
			end
		end,

		Enabled = function()
			eventFrame:SetScript("OnUpdate", onUpdate);
		end,
	};

	addon.System.Callbacks.RegisterAddonCallbacks(moduleEvents);
end