-- Pulls back the Addon-Local Variables and store them locally.
local addonName, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];
local L = LibStub("AceLocale-3.0"):GetLocale(addonName);

local pluginName = "SpellFlash";
local pluginDescription = "This plugin allows the usage of Spell Flash for rotation suggestions instead of Weak Auras.\n\nThis plugin will not interfere with the Weak Auras plugin so it is safe to use both if you wish.\n\nOriginally Designed by Baturios.\nUpdated to plugin by LunaEclipse.";

-- Check for SpellFlashCore and LunaEclipse Rotation Helper being loaded first so it doesn't register the plugin if its not needed.
if addon then
    local optionsTable = {
        type = "group",
        name = pluginName,
        childGroups = "tree",
        args = {
            groupColors = {
                type = "group",
                name = addon.Options.Core.CreaterHeader(L["Colors"]),
                get = function(info)
                    local returnValue;

                    if info then
                        local Tables = addon.Core.Tables;

                        local sectionName = info[#info - 1];
                        local keyName = info[#info];
                        local profile = Tables.GetValue(addonTable.spellFlashSettings, sectionName);

                        returnValue = Tables.GetValue(profile, keyName);
                    end

                    return returnValue.r, returnValue.g, returnValue.b;
                end,
                set = function(info, r, g, b)
                    if info then
                        local Tables = addon.Core.Tables;

                        local sectionName = info[#info - 1];
                        local keyName = info[#info];
                        local profile = Tables.GetValue(addonTable.spellFlashSettings, sectionName, {});

                        profile[keyName] = { r = r, g = g, b = b };

                        -- Put the settings into the database
                        addonTable.spellFlashSettings[sectionName] = profile;
                    end
                end,
                order = 10,
                inline = true,
                args = {
                    colorSingleTarget = {
                        type = "color",
                        name = L["Single Target Spell"],
                        desc = L["Color of flash for single target"],
                        order = 10,
                    },
                    colorMultiTarget = {
                        type = "color",
                        name = L["AOE Spell"],
                        desc = L["Color of flash for AOE"],
                        order = 20,
                    },
                    colorDefensive = {
                        type = "color",
                        name = L["Defensive Spell"],
                        desc = L["Color of flash for defensive spell"],
                        order = 30,
                    },
                    colorInterrupt = {
                        type = "color",
                        name = L["Interrupt"],
                        desc = L["Color of flash for interruption"],
                        order = 40,
                    },
                    colorCooldown = {
                        type = "color",
                        name = L["Cooldown Spell"],
                        desc = L["Color of flash for CD"],
                        order = 50,
                    },
                    colorOFFGCD = {
                        type = "color",
                        name = L["OFFCD Spell"],
                        desc = L["Color of flash for OFFCD"],
                        order = 60,
                    },
                },
            },
        },
    };

    -- The plugin still has to be registered even if its not loaded into memory, otherwise there is no checkbox to re-enable it.
    addon.System.Plugins.RegisterPlugin(pluginName, pluginDescription, optionsTable);
end