-- Pulls back the Addon-Local Variables and store them locally.
local addonName, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];

-- Check for LunaEclipse Rotation Helper being loaded first, there is no point doing anything else if its not loaded.
if addon then
	local defaultAddonSettings = {
		allowZeroMacros = 0,
		autoLearnTalents = 0,
		styleSelection = "Ovale",
		versionWeakAura = 0,
	};

	local defaultCharacterSettings = {
		versionMacroProfile = 0,
	};

	local defaultEquipManager = {
		useOverlay = 1,
		disableDowngrade = 1,
		downgradeLevel = 25,
		Sets = {},
	};

	local defaultGameTweaks = {
		useMemoryCleaner = 1,
		cleanupRate = 30,
	};

	local defaultInterruptManager = {
		InterruptOptions = {
			limitFriendlyParty = 0,
			limitFriendlyRaid = 0,
			limitFriendlySolo = 0,
			limitPetParty = 0,
			limitPetRaid = 0,
			limitPetSolo = 0,
			limitPlayerParty = 0,
			limitPlayerRaid = 0,
			limitPlayerSolo = 0,
			noLimitsParty = 1,
			noLimitsRaid = 1,
			noLimitsSolo = 1,
		},
		requireRotation = 1,
		useCC = 0,
		useStun = 1,
	};

	local defaultMacroProfile = {};
	local defaultMinimapSettings = {
		changeLootSpec = 1,
		changeSpec = 1,
		hide = false,
		reloadUI = 1,
		toggleIcon = 1,
		useRotations = 1,
		useTalentBuilds = 1,
	};

	local defaultSpellFlash = {
		groupColors = {
			colorCooldown = { r = 0, g = 1, b = 0 },
			colorDefensive = { r = 1.0, g = 0.65, b = 0 },
			colorInterrupt = { r = 1.0, g = 1.0, b = 1.0 },
			colorMultiTarget = { r = 1.0, g = 0, b = 0 },
			colorOFFGCD = { r = 0, g = 1.0, b = 1.0 },
			colorSingleTarget = { r = 1.0, g = 1.0, b = 0 },
		},
	};

	local defaultTalentBuilds = {
		CurrentBuild = {},
		CustomBuilds = {},
	};

	local defaultWeakAuraSettings = {
		CustomAuras = {},
		Styles = {
			LunaEclipse = {
				LERH_AOE = 1,
				LERH_Bloodlust = 0,
				LERH_CD = 1,
				LERH_DamageAll = 0,
				LERH_DamageMagic = 0,
				LERH_DamagePhysical = 0,
				LERH_Defensive = 1,
				LERH_Enemies = 0,
				LERH_Interrupt = 1,
				LERH_OffGCD = 1,
				LERH_Range = 0,
				LERH_TimeToDie = 0,
			},
			Ovale = {
				LERH_AOE = 1,
				LERH_Bloodlust = 0,
				LERH_CD = 1,
				LERH_DamageAll = 0,
				LERH_DamageMagic = 0,
				LERH_DamagePhysical = 0,
				LERH_Defensive = 1,
				LERH_Enemies = 0,
				LERH_Interrupt = 1,
				LERH_OffGCD = 1,
				LERH_Range = 0,
				LERH_ST = 1,
				LERH_TimeToDie = 0,
			},
		},
	};

	-- Color Tags
	addonTable.COLORS = {
		CLASS = {
			[1] = "|cFFC79C6E",
			[2] = "|cFFF58CBA",
			[3] = "|cFFABD473",
			[4] = "|cFFFFF569",
			[5] = "|cFFFFFFFF",
			[6] = "|cFFC41F3B",
			[7] = "|cFF0070DE",
			[8] = "|cFF69CCF0",
			[9] = "|cFF9482C9",
			[10] = "|cFF00FF96",
			[11] = "|cFFFF7D0A",
			[12] = "|cFFA330C9",
		},

		TAGS = {
			ACTIVE = "|cFF00FF00",
			DEFAULT = "|cFF7FCCFF",
			HIGHLIGHT = "|cFF1784D1",
			INACTIVE = "|cFFFF0000",
			MESSAGE = "|cFFFFFF00",
		},
	};

	-- General Tags
	addonTable.TAGS = {
		END = "|r",
	};

	-- Table for holding Talent Build data.
	addonTable.TalentBuilds = {
		Builds = {},
		Custom = "Custom Build",
		MaxLevel = 120,
	};

	-- Internal namespace for private WeakAuras functions and data.
	addonTable.WeakAuras = {
		Core = {},
		ForcedRefresh = 6,
		Styles = {},
		Templates = {},
		Version = 6,
	};

	-- Tables to hold private functions.
	local profileVersion = 4;
	local variableError = false;

	local moduleEvents = {
		addonLoaded = function(name)
			if name == addonName then
				local coreSettings = addon.Core.Settings;
				local coreTables = addon.Core.Tables;

				if coreTables.GetValue(LERH_CharacterSettings, "InterruptOptions") and not coreTables.GetValue(LERH_InterruptManager, "InterruptOptions") then
					defaultInterruptManager.InterruptOptions = coreTables.Duplicate(LERH_CharacterSettings.InterruptOptions);

					-- Okay we have imported the old settings from the rotation addon to the plugin, so remove them from the addon settings, to do this leave the last 2 parameters nil.
					coreSettings.SetCharacterValue("InterruptOptions");
				end

				addonTable.addonSettings = coreTables.ApplyDefaults(LERH_Features_AddonSettings, defaultAddonSettings);
				addonTable.characterSettings = coreTables.ApplyDefaults(LERH_Features_CharacterSettings, defaultCharacterSettings);
				addonTable.equipManager = coreTables.ApplyDefaults(LERH_EquipManager, defaultEquipManager);
				addonTable.gameTweaks = coreTables.ApplyDefaults(LERH_GameTweaks, defaultGameTweaks);
				addonTable.interruptManager = coreTables.ApplyDefaults(LERH_InterruptManager, defaultInterruptManager);
				addonTable.macroProfiles = coreTables.ApplyDefaults(LERH_MacroProfiles, defaultMacroProfile);
				addonTable.minimapSettings = coreTables.ApplyDefaults(LERH_Minimap, defaultMinimapSettings);
				addonTable.spellFlashSettings = coreTables.ApplyDefaults(LERH_SpellFlash, defaultSpellFlash);
				addonTable.talentBuilds = coreTables.ApplyDefaults(LERH_TalentBuilds, defaultTalentBuilds);
				addonTable.weakAuraSettings = coreTables.ApplyDefaults(LERH_WeakAuras, defaultWeakAuraSettings);

				if LESM_MacroProfile and coreTables.IsEmpty(addonTable.macroProfiles) then
					addonTable.macroProfiles = LESM_MacroProfile.profiles.Settings;
				end

				if addon.System.Plugins.IsActive("Macro Manager") and coreTables.GetValue(addonTable.characterSettings, "versionMacroProfile", 0) < profileVersion then
					addonTable.macroProfiles = addon.Features.MacroManager.Update(addonTable.macroProfiles);
					addonTable.characterSettings.versionMacroProfile = profileVersion;
				end
			end
		end,

		errorSavedVariables = function(name)
			if name == addonName then
				variableError = true;
			end
		end,

		playerLogout = function()
			if not variableError then
				LERH_Features_AddonSettings = addonTable.addonSettings;
				LERH_Features_CharacterSettings = addonTable.characterSettings;
				LERH_EquipManager = addonTable.equipManager;
				LERH_GameTweaks = addonTable.gameTweaks;
				LERH_InterruptManager = addonTable.interruptManager;
				LERH_MacroProfiles = addonTable.macroProfiles;
				LERH_Minimap = addonTable.minimapSettings;
				LERH_SpellFlash = addonTable.spellFlashSettings;
				LERH_TalentBuilds = addonTable.talentBuilds;
				LERH_WeakAuras = addonTable.weakAuraSettings;
			end
		end,
	};

	addon.System.Callbacks.RegisterAddonCallbacks(moduleEvents);
end